from _typeshed import SupportsRichComparison
from collections import OrderedDict
from collections.abc import Callable
from typing import IO, Any, TypeVar, overload
from typing_extensions import TypeAlias

from simplejson.decoder import JSONDecoder as JSONDecoder
from simplejson.encoder import JSONEncoder as JSONEncoder, JSONEncoderForHTML as JSONEncoderForHTML
from simplejson.raw_json import RawJSON as RawJSON
from simplejson.scanner import JSONDecodeError as JSONDecodeError

_LoadsString: TypeAlias = str | bytes | bytearray
_T = TypeVar("_T")

@overload
def dumps(
    obj: Any,
    skipkeys: bool = False,
    ensure_ascii: bool = True,
    check_circular: bool = True,
    allow_nan: bool = False,
    *,
    cls: type[JSONEncoder],
    indent: str | int | None = None,
    separators: tuple[str, str] | None = None,
    encoding: str | None = "utf-8",
    default: Callable[[Any], Any] | None = None,
    use_decimal: bool = True,
    namedtuple_as_object: bool = True,
    tuple_as_array: bool = True,
    bigint_as_string: bool = False,
    sort_keys: bool = False,
    item_sort_key: Callable[[Any], SupportsRichComparison] | None = None,
    for_json: bool = False,
    ignore_nan: bool = False,
    int_as_string_bitcount: int | None = None,
    iterable_as_array: bool = False,
    **kw: Any,
) -> str: ...
@overload
def dumps(
    obj: Any,
    skipkeys: bool = False,
    ensure_ascii: bool = True,
    check_circular: bool = True,
    allow_nan: bool = False,
    cls: type[JSONEncoder] | None = None,
    indent: str | int | None = None,
    separators: tuple[str, str] | None = None,
    encoding: str | None = "utf-8",
    default: Callable[[Any], Any] | None = None,
    use_decimal: bool = True,
    namedtuple_as_object: bool = True,
    tuple_as_array: bool = True,
    bigint_as_string: bool = False,
    sort_keys: bool = False,
    item_sort_key: Callable[[Any], SupportsRichComparison] | None = None,
    for_json: bool = False,
    ignore_nan: bool = False,
    int_as_string_bitcount: int | None = None,
    iterable_as_array: bool = False,
) -> str: ...
@overload
def dump(
    obj: Any,
    fp: IO[str],
    skipkeys: bool = False,
    ensure_ascii: bool = True,
    check_circular: bool = True,
    allow_nan: bool = False,
    *,
    cls: type[JSONEncoder],
    indent: str | int | None = None,
    separators: tuple[str, str] | None = None,
    encoding: str | None = "utf-8",
    default: Callable[[Any], Any] | None = None,
    use_decimal: bool = True,
    namedtuple_as_object: bool = True,
    tuple_as_array: bool = True,
    bigint_as_string: bool = False,
    sort_keys: bool = False,
    item_sort_key: Callable[[Any], SupportsRichComparison] | None = None,
    for_json: bool = False,
    ignore_nan: bool = False,
    int_as_string_bitcount: int | None = None,
    iterable_as_array: bool = False,
    **kw: Any,
) -> None: ...
@overload
def dump(
    obj: Any,
    fp: IO[str],
    skipkeys: bool = False,
    ensure_ascii: bool = True,
    check_circular: bool = True,
    allow_nan: bool = False,
    cls: type[JSONEncoder] | None = None,
    indent: str | int | None = None,
    separators: tuple[str, str] | None = None,
    encoding: str | None = "utf-8",
    default: Callable[[Any], Any] | None = None,
    use_decimal: bool = True,
    namedtuple_as_object: bool = True,
    tuple_as_array: bool = True,
    bigint_as_string: bool = False,
    sort_keys: bool = False,
    item_sort_key: Callable[[Any], SupportsRichComparison] | None = None,
    for_json: bool = False,
    ignore_nan: bool = False,
    int_as_string_bitcount: int | None = None,
    iterable_as_array: bool = False,
) -> None: ...
@overload
def loads(
    s: _LoadsString,
    encoding: str | None = None,
    *,
    cls: type[JSONDecoder],
    object_hook: Callable[[dict[Any, Any]], Any] | None = None,
    parse_float: Callable[[str], Any] | None = None,
    parse_int: Callable[[str], Any] | None = None,
    parse_constant: Callable[[str], Any] | None = None,
    object_pairs_hook: Callable[[list[tuple[Any, Any]]], Any] | None = None,
    use_decimal: bool = False,
    allow_nan: bool = False,
    **kw: Any,
) -> Any: ...
@overload
def loads(
    s: _LoadsString,
    encoding: str | None = None,
    cls: type[JSONDecoder] | None = None,
    object_hook: Callable[[dict[Any, Any]], Any] | None = None,
    parse_float: Callable[[str], Any] | None = None,
    parse_int: Callable[[str], Any] | None = None,
    parse_constant: Callable[[str], Any] | None = None,
    object_pairs_hook: Callable[[list[tuple[Any, Any]]], Any] | None = None,
    use_decimal: bool = False,
    allow_nan: bool = False,
) -> Any: ...
@overload
def load(
    fp: IO[str],
    encoding: str | None = None,
    *,
    cls: type[JSONDecoder],
    object_hook: Callable[[dict[Any, Any]], Any] | None = None,
    parse_float: Callable[[str], Any] | None = None,
    parse_int: Callable[[str], Any] | None = None,
    parse_constant: Callable[[str], Any] | None = None,
    object_pairs_hook: Callable[[list[tuple[Any, Any]]], Any] | None = None,
    use_decimal: bool = False,
    allow_nan: bool = False,
    **kw: Any,
) -> Any: ...
@overload
def load(
    fp: IO[str],
    encoding: str | None = None,
    cls: type[JSONDecoder] | None = None,
    object_hook: Callable[[dict[Any, Any]], Any] | None = None,
    parse_float: Callable[[str], Any] | None = None,
    parse_int: Callable[[str], Any] | None = None,
    parse_constant: Callable[[str], Any] | None = None,
    object_pairs_hook: Callable[[list[tuple[Any, Any]]], Any] | None = None,
    use_decimal: bool = False,
    allow_nan: bool = False,
) -> Any: ...
def simple_first(kv: tuple[_T, object]) -> tuple[bool, _T]: ...

__all__ = [
    "dump",
    "dumps",
    "load",
    "loads",
    "JSONDecoder",
    "JSONDecodeError",
    "JSONEncoder",
    "OrderedDict",
    "simple_first",
    "RawJSON",
]



from _typeshed import Incomplete
from logging import Logger

log: Logger

class BaseApp:
    client_cls: Incomplete
    OAUTH_APP_CONFIG: Incomplete
    def request(self, method, url, token=None, **kwargs): ...
    def get(self, url, **kwargs): ...
    def post(self, url, **kwargs): ...
    def patch(self, url, **kwargs): ...
    def put(self, url, **kwargs): ...
    def delete(self, url, **kwargs): ...

class _RequestMixin: ...

class OAuth1Base:
    client_cls: Incomplete
    framework: Incomplete
    name: Incomplete
    client_id: Incomplete
    client_secret: Incomplete
    request_token_url: Incomplete
    request_token_params: Incomplete
    access_token_url: Incomplete
    access_token_params: Incomplete
    authorize_url: Incomplete
    authorize_params: Incomplete
    api_base_url: Incomplete
    client_kwargs: Incomplete
    def __init__(
        self,
        framework,
        name=None,
        fetch_token=None,
        client_id=None,
        client_secret=None,
        request_token_url=None,
        request_token_params=None,
        access_token_url=None,
        access_token_params=None,
        authorize_url=None,
        authorize_params=None,
        api_base_url=None,
        client_kwargs=None,
        user_agent=None,
        **kwargs,
    ) -> None: ...

class OAuth1Mixin(_RequestMixin, OAuth1Base):
    def request(self, method, url, token=None, **kwargs): ...
    def create_authorization_url(self, redirect_uri=None, **kwargs): ...
    def fetch_access_token(self, request_token=None, **kwargs): ...

class OAuth2Base:
    client_cls: Incomplete
    framework: Incomplete
    name: Incomplete
    client_id: Incomplete
    client_secret: Incomplete
    access_token_url: Incomplete
    access_token_params: Incomplete
    authorize_url: Incomplete
    authorize_params: Incomplete
    api_base_url: Incomplete
    client_kwargs: Incomplete
    compliance_fix: Incomplete
    client_auth_methods: Incomplete
    server_metadata: Incomplete
    def __init__(
        self,
        framework,
        name=None,
        fetch_token=None,
        update_token=None,
        client_id=None,
        client_secret=None,
        access_token_url=None,
        access_token_params=None,
        authorize_url=None,
        authorize_params=None,
        api_base_url=None,
        client_kwargs=None,
        server_metadata_url=None,
        compliance_fix=None,
        client_auth_methods=None,
        user_agent=None,
        **kwargs,
    ) -> None: ...

class OAuth2Mixin(_RequestMixin, OAuth2Base):
    def request(self, method, url, token=None, **kwargs): ...
    def load_server_metadata(self): ...
    def create_authorization_url(self, redirect_uri=None, **kwargs): ...
    def fetch_access_token(self, redirect_uri=None, **kwargs): ...



from _typeshed import Incomplete
from logging import Logger

from oauthlib.common import Request

from ..tokens import TokenBase
from .base import GrantTypeBase

log: Logger

def code_challenge_method_s256(verifier: str, challenge: str) -> bool: ...
def code_challenge_method_plain(verifier: str, challenge: str) -> bool: ...

class AuthorizationCodeGrant(GrantTypeBase):
    default_response_mode: str
    response_types: list[str]
    def create_authorization_code(self, request: Request) -> dict[str, str]: ...
    def create_authorization_response(
        self, request: Request, token_handler: TokenBase
    ) -> tuple[dict[str, str], None, int | None]: ...
    def create_token_response(self, request: Request, token_handler: TokenBase) -> tuple[dict[str, str], str, int | None]: ...
    def validate_authorization_request(self, request: Request) -> tuple[Incomplete, dict[str, Incomplete]]: ...
    def validate_token_request(self, request: Request) -> None: ...
    def validate_code_challenge(self, challenge: str, challenge_method: str, verifier: str) -> bool: ...



import os
from _typeshed import Incomplete, SupportsGetItem, SupportsKeysAndGetItem
from collections.abc import Iterable, Mapping
from typing import Any, Final

from ..geodataframe import GeoDataFrame

METADATA_VERSION: Final[str]
SUPPORTED_VERSIONS_LITERAL = Incomplete
SUPPORTED_VERSIONS: Final[list[str]]
GEOARROW_ENCODINGS: Final[list[str]]
SUPPORTED_ENCODINGS: Final[list[str]]
PARQUET_GEOMETRY_ENCODINGS = Incomplete

def _read_parquet(
    path: str | os.PathLike[str],
    columns: Iterable[str] | None = None,
    storage_options: SupportsKeysAndGetItem[str, Any] | None = None,  # type depend on the connection
    bbox: SupportsGetItem[int, float] | None = None,
    to_pandas_kwargs: Mapping[str, Incomplete] | None = None,
    **kwargs,  # kwargs passed to pyarrow.parquet.read_table
) -> GeoDataFrame: ...
def _read_feather(
    path: str | os.PathLike[str],
    columns: Iterable[str] | None = None,
    to_pandas_kwargs: Mapping[str, Incomplete] | None = None,
    **kwargs,  # kwargs passed to pyarrow.feather.read_table
) -> GeoDataFrame: ...



from _typeshed import Incomplete
from dataclasses import dataclass
from typing import Final
from typing_extensions import Self

from networkx.classes.digraph import DiGraph
from networkx.classes.graph import _Node
from networkx.utils.backends import _dispatchable
from numpy.random import RandomState

__all__ = [
    "branching_weight",
    "greedy_branching",
    "maximum_branching",
    "minimum_branching",
    "minimal_branching",
    "maximum_spanning_arborescence",
    "minimum_spanning_arborescence",
    "ArborescenceIterator",
]

KINDS: Final[set[str]]
STYLES: Final[dict[str, str]]
INF: Final[float]

def random_string(L=15, seed=None): ...
@_dispatchable
def branching_weight(G: DiGraph[_Node], attr: str = "weight", default: float = 1): ...
@_dispatchable
def greedy_branching(
    G: DiGraph[_Node], attr: str = "weight", default: float = 1, kind: str = "max", seed: int | RandomState | None = None
): ...
@_dispatchable
def maximum_branching(
    G: DiGraph[_Node], attr: str = "weight", default: float = 1, preserve_attrs: bool = False, partition: str | None = None
): ...
@_dispatchable
def minimum_branching(
    G: DiGraph[_Node], attr: str = "weight", default: float = 1, preserve_attrs: bool = False, partition: str | None = None
): ...
@_dispatchable
def minimal_branching(G: DiGraph[_Node], /, *, attr="weight", default=1, preserve_attrs=False, partition=None): ...
@_dispatchable
def maximum_spanning_arborescence(
    G: DiGraph[_Node], attr: str = "weight", default: float = 1, preserve_attrs: bool = False, partition: str | None = None
): ...
@_dispatchable
def minimum_spanning_arborescence(
    G: DiGraph[_Node], attr: str = "weight", default: float = 1, preserve_attrs: bool = False, partition: str | None = None
): ...

class ArborescenceIterator:
    @dataclass(order=True)
    class Partition:
        mst_weight: float
        partition_dict: dict[Incomplete, Incomplete]
        def __copy__(self) -> ArborescenceIterator.Partition: ...

    G: Incomplete
    weight: Incomplete
    minimum: Incomplete
    method: Incomplete
    partition_key: str
    init_partition: Incomplete

    def __init__(self, G: DiGraph[_Node], weight: str = "weight", minimum: bool = True, init_partition=None) -> None: ...
    partition_queue: Incomplete

    def __iter__(self) -> Self: ...
    def __next__(self): ...



from _typeshed import BytesPath, Incomplete, StrPath, Unused
from collections.abc import Callable, Iterable, MutableSequence, Sequence
from typing import ClassVar, Final, Literal, TypeVar, overload
from typing_extensions import TypeAlias, TypeVarTuple, Unpack

_Macro: TypeAlias = tuple[str] | tuple[str, str | None]
_StrPathT = TypeVar("_StrPathT", bound=StrPath)
_BytesPathT = TypeVar("_BytesPathT", bound=BytesPath)
_Ts = TypeVarTuple("_Ts")

class Compiler:
    compiler_type: ClassVar[str]
    executables: ClassVar[dict[str, Incomplete]]

    # Subclasses that rely on the standard filename generation methods
    # implemented below should override these
    src_extensions: ClassVar[list[str] | None]
    obj_extension: ClassVar[str | None]
    static_lib_extension: ClassVar[str | None]
    shared_lib_extension: ClassVar[str | None]
    static_lib_format: ClassVar[str | None]
    shared_lib_format: ClassVar[str | None]
    exe_extension: ClassVar[str | None]

    language_map: ClassVar[dict[str, str]]
    language_order: ClassVar[list[str]]
    dry_run: bool
    force: bool
    verbose: bool
    output_dir: str | None
    macros: list[_Macro]
    include_dirs: list[str]
    libraries: list[str]
    library_dirs: list[str]
    runtime_library_dirs: list[str]
    objects: list[str]

    SHARED_OBJECT: Final = "shared_object"
    SHARED_LIBRARY: Final = "shared_library"
    EXECUTABLE: Final = "executable"
    def __init__(self, verbose: bool = False, dry_run: bool = False, force: bool = False) -> None: ...
    def add_include_dir(self, dir: str) -> None: ...
    def set_include_dirs(self, dirs: list[str]) -> None: ...
    def add_library(self, libname: str) -> None: ...
    def set_libraries(self, libnames: list[str]) -> None: ...
    def add_library_dir(self, dir: str) -> None: ...
    def set_library_dirs(self, dirs: list[str]) -> None: ...
    def add_runtime_library_dir(self, dir: str) -> None: ...
    def set_runtime_library_dirs(self, dirs: list[str]) -> None: ...
    def define_macro(self, name: str, value: str | None = None) -> None: ...
    def undefine_macro(self, name: str) -> None: ...
    def add_link_object(self, object: str) -> None: ...
    def set_link_objects(self, objects: list[str]) -> None: ...
    def detect_language(self, sources: str | list[str]) -> str | None: ...
    def find_library_file(self, dirs: Iterable[str], lib: str, debug: bool = False) -> str | None: ...
    def has_function(
        self,
        funcname: str,
        includes: Iterable[str] | None = None,
        include_dirs: list[str] | tuple[str, ...] | None = None,
        libraries: list[str] | None = None,
        library_dirs: list[str] | tuple[str, ...] | None = None,
    ) -> bool: ...
    def library_dir_option(self, dir: str) -> str: ...
    def library_option(self, lib: str) -> str: ...
    def runtime_library_dir_option(self, dir: str) -> str: ...
    def set_executables(self, **kwargs: str) -> None: ...
    def set_executable(self, key: str, value) -> None: ...
    def compile(
        self,
        sources: Sequence[StrPath],
        output_dir: str | None = None,
        macros: list[_Macro] | None = None,
        include_dirs: list[str] | tuple[str, ...] | None = None,
        debug: bool = False,
        extra_preargs: list[str] | None = None,
        extra_postargs: list[str] | None = None,
        depends: list[str] | tuple[str, ...] | None = None,
    ) -> list[str]: ...
    def create_static_lib(
        self,
        objects: list[str] | tuple[str, ...],
        output_libname: str,
        output_dir: str | None = None,
        debug: bool = False,
        target_lang: str | None = None,
    ) -> None: ...
    def link(
        self,
        target_desc: str,
        objects: list[str] | tuple[str, ...],
        output_filename: str,
        output_dir: str | None = None,
        libraries: list[str] | tuple[str, ...] | None = None,
        library_dirs: list[str] | tuple[str, ...] | None = None,
        runtime_library_dirs: list[str] | tuple[str, ...] | None = None,
        export_symbols: Iterable[str] | None = None,
        debug: bool = False,
        extra_preargs: list[str] | None = None,
        extra_postargs: list[str] | None = None,
        build_temp: StrPath | None = None,
        target_lang: str | None = None,
    ) -> None: ...
    def link_executable(
        self,
        objects: list[str] | tuple[str, ...],
        output_progname: str,
        output_dir: str | None = None,
        libraries: list[str] | tuple[str, ...] | None = None,
        library_dirs: list[str] | tuple[str, ...] | None = None,
        runtime_library_dirs: list[str] | tuple[str, ...] | None = None,
        debug: bool = False,
        extra_preargs: list[str] | None = None,
        extra_postargs: list[str] | None = None,
        target_lang: str | None = None,
    ) -> None: ...
    def link_shared_lib(
        self,
        objects: list[str] | tuple[str, ...],
        output_libname: str,
        output_dir: str | None = None,
        libraries: list[str] | tuple[str, ...] | None = None,
        library_dirs: list[str] | tuple[str, ...] | None = None,
        runtime_library_dirs: list[str] | tuple[str, ...] | None = None,
        export_symbols: Iterable[str] | None = None,
        debug: bool = False,
        extra_preargs: list[str] | None = None,
        extra_postargs: list[str] | None = None,
        build_temp: StrPath | None = None,
        target_lang: str | None = None,
    ) -> None: ...
    def link_shared_object(
        self,
        objects: list[str] | tuple[str, ...],
        output_filename: str,
        output_dir: str | None = None,
        libraries: list[str] | tuple[str, ...] | None = None,
        library_dirs: list[str] | tuple[str, ...] | None = None,
        runtime_library_dirs: list[str] | tuple[str, ...] | None = None,
        export_symbols: Iterable[str] | None = None,
        debug: bool = False,
        extra_preargs: list[str] | None = None,
        extra_postargs: list[str] | None = None,
        build_temp: StrPath | None = None,
        target_lang: str | None = None,
    ) -> None: ...
    def preprocess(
        self,
        source: StrPath,
        output_file: StrPath | None = None,
        macros: list[_Macro] | None = None,
        include_dirs: list[str] | tuple[str, ...] | None = None,
        extra_preargs: list[str] | None = None,
        extra_postargs: Iterable[str] | None = None,
    ) -> None: ...
    @overload
    def executable_filename(self, basename: str, strip_dir: Literal[False] = False, output_dir: StrPath = "") -> str: ...
    @overload
    def executable_filename(self, basename: StrPath, strip_dir: Literal[True], output_dir: StrPath = "") -> str: ...
    def library_filename(
        self, libname: str, lib_type: str = "static", strip_dir: bool = False, output_dir: StrPath = ""
    ) -> str: ...
    @property
    def out_extensions(self) -> dict[str, str]: ...
    def object_filenames(
        self, source_filenames: Iterable[StrPath], strip_dir: bool = False, output_dir: StrPath | None = ""
    ) -> list[str]: ...
    @overload
    def shared_object_filename(self, basename: str, strip_dir: Literal[False] = False, output_dir: StrPath = "") -> str: ...
    @overload
    def shared_object_filename(self, basename: StrPath, strip_dir: Literal[True], output_dir: StrPath = "") -> str: ...
    def execute(
        self, func: Callable[[Unpack[_Ts]], Unused], args: tuple[Unpack[_Ts]], msg: str | None = None, level: int = 1
    ) -> None: ...
    def spawn(self, cmd: MutableSequence[bytes | StrPath]) -> None: ...
    def mkpath(self, name: str, mode: int = 0o777) -> None: ...
    @overload
    def move_file(self, src: StrPath, dst: _StrPathT) -> _StrPathT | str: ...
    @overload
    def move_file(self, src: BytesPath, dst: _BytesPathT) -> _BytesPathT | bytes: ...
    def announce(self, msg: str, level: int = 1) -> None: ...
    def warn(self, msg: str) -> None: ...
    def debug_print(self, msg: str) -> None: ...

def get_default_compiler(osname: str | None = None, platform: str | None = None) -> str: ...

compiler_class: dict[str, tuple[str, str, str]]

def show_compilers() -> None: ...
def new_compiler(
    plat: str | None = None, compiler: str | None = None, verbose: bool = False, dry_run: bool = False, force: bool = False
) -> Compiler: ...
def gen_preprocess_options(macros: Iterable[_Macro], include_dirs: Iterable[str]) -> list[str]: ...
def gen_lib_options(
    compiler: Compiler, library_dirs: Iterable[str], runtime_library_dirs: Iterable[str], libraries: Iterable[str]
) -> list[str]: ...



from collections.abc import Awaitable, Callable
from typing import Any, Protocol, type_check_only
from typing_extensions import TypeAlias

from asgiref.typing import ASGIApplication, ASGIReceiveCallable

def name_that_thing(thing: object) -> str: ...
async def await_many_dispatch(
    consumer_callables: list[Callable[[], Awaitable[ASGIReceiveCallable]]], dispatch: Callable[[dict[str, Any]], Awaitable[None]]
) -> None: ...

# Defines a generic ASGI middleware protocol.
# All arguments are typed as `Any` to maximize compatibility with third-party ASGI middleware
# that may not strictly follow type conventions or use more specific signatures.
@type_check_only
class _MiddlewareProtocol(Protocol):
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    async def __call__(self, scope: Any, receive: Any, send: Any) -> Any: ...

_ChannelApplication: TypeAlias = _MiddlewareProtocol | ASGIApplication  # noqa: Y047



from typing import Final, Literal

EPOCH: Final = 1948320.5
WEEKDAYS: Final[tuple[str, ...]]
MONTHS: Final[list[str]]
HAS_31_DAYS: Final = (1, 2, 3, 4, 5, 6)
HAS_30_DAYS: Final = (7, 8, 9, 10, 11)

def leap(year: int) -> bool: ...
def equinox_jd(gyear: int) -> int: ...
def last_equinox_jd(jd: float) -> int: ...
def jd_to_pyear(jd: float) -> tuple[int, int]: ...
def to_jd(year: int, month: int, day: int) -> float: ...
def from_jd(jd: float) -> tuple[int, int, int]: ...
def from_gregorian(year: int, month: int, day: int) -> tuple[int, int, int]: ...
def to_gregorian(year: int, month: int, day: int) -> tuple[int, int, int]: ...
def month_length(year: int, month: int) -> Literal[29, 30, 31]: ...
def monthcalendar(year: int, month: int) -> list[list[int | None]]: ...
def format(year: int, month: int, day: int) -> str: ...



from _typeshed import Incomplete
from collections import OrderedDict
from typing import Protocol, TypeVar, overload, type_check_only

_T = TypeVar("_T")
_VT = TypeVar("_VT")

@type_check_only
class _SupportsCasefold(Protocol):
    def casefold(self) -> str: ...

@type_check_only
class _SupportsLower(Protocol):
    def lower(self) -> str: ...

class CaseFoldedOrderedDict(OrderedDict[str, _VT]):
    def __getitem__(self, key: _SupportsCasefold | _SupportsLower) -> _VT: ...
    def __setitem__(self, key: _SupportsCasefold | _SupportsLower, value: _VT) -> None: ...
    def __contains__(self, key: _SupportsCasefold | _SupportsLower) -> bool: ...  # type: ignore[override]
    # See overloads for dict class:
    @overload
    def get(self, key: _SupportsCasefold | _SupportsLower, default: None = None) -> _VT | None: ...
    @overload
    def get(self, key: _SupportsCasefold | _SupportsLower, default: _VT) -> _VT: ...
    @overload
    def get(self, key: _SupportsCasefold | _SupportsLower, default: _T) -> _VT | _T: ...
    @overload  # type: ignore[override]
    def pop(self, key: _SupportsCasefold | _SupportsLower, default: None = None) -> _VT | None: ...
    @overload
    def pop(self, key: _SupportsCasefold | _SupportsLower, default: _VT) -> _VT: ...
    @overload
    def pop(self, key: _SupportsCasefold | _SupportsLower, default: _T) -> _VT | _T: ...

def parse(value: str) -> CaseFoldedOrderedDict[Incomplete]: ...



from .alabama import (
    Alabama as Alabama,
    AlabamaBaldwinCounty as AlabamaBaldwinCounty,
    AlabamaMobileCounty as AlabamaMobileCounty,
    AlabamaPerryCounty as AlabamaPerryCounty,
)
from .alaska import Alaska as Alaska
from .american_samoa import AmericanSamoa as AmericanSamoa
from .arizona import Arizona as Arizona
from .arkansas import Arkansas as Arkansas
from .california import (
    California as California,
    CaliforniaBerkeley as CaliforniaBerkeley,
    CaliforniaEducation as CaliforniaEducation,
    CaliforniaSanFrancisco as CaliforniaSanFrancisco,
    CaliforniaWestHollywood as CaliforniaWestHollywood,
)
from .colorado import Colorado as Colorado
from .connecticut import Connecticut as Connecticut
from .core import FederalReserveSystem as FederalReserveSystem, UnitedStates as UnitedStates
from .delaware import Delaware as Delaware
from .district_columbia import DistrictOfColumbia as DistrictOfColumbia
from .florida import (
    Florida as Florida,
    FloridaCircuitCourts as FloridaCircuitCourts,
    FloridaLegal as FloridaLegal,
    FloridaMiamiDade as FloridaMiamiDade,
)
from .georgia import Georgia as Georgia
from .guam import Guam as Guam
from .hawaii import Hawaii as Hawaii
from .idaho import Idaho as Idaho
from .illinois import ChicagoIllinois as ChicagoIllinois, Illinois as Illinois
from .indiana import Indiana as Indiana
from .iowa import Iowa as Iowa
from .kansas import Kansas as Kansas
from .kentucky import Kentucky as Kentucky
from .louisiana import Louisiana as Louisiana
from .maine import Maine as Maine
from .maryland import Maryland as Maryland
from .massachusetts import Massachusetts as Massachusetts, SuffolkCountyMassachusetts as SuffolkCountyMassachusetts
from .michigan import Michigan as Michigan
from .minnesota import Minnesota as Minnesota
from .mississippi import Mississippi as Mississippi
from .missouri import Missouri as Missouri
from .montana import Montana as Montana
from .nebraska import Nebraska as Nebraska
from .nevada import Nevada as Nevada
from .new_hampshire import NewHampshire as NewHampshire
from .new_jersey import NewJersey as NewJersey
from .new_mexico import NewMexico as NewMexico
from .new_york import NewYork as NewYork
from .north_carolina import NorthCarolina as NorthCarolina
from .north_dakota import NorthDakota as NorthDakota
from .ohio import Ohio as Ohio
from .oklahoma import Oklahoma as Oklahoma
from .oregon import Oregon as Oregon
from .pennsylvania import Pennsylvania as Pennsylvania
from .rhode_island import RhodeIsland as RhodeIsland
from .south_carolina import SouthCarolina as SouthCarolina
from .south_dakota import SouthDakota as SouthDakota
from .tennessee import Tennessee as Tennessee
from .texas import Texas as Texas, TexasBase as TexasBase
from .utah import Utah as Utah
from .vermont import Vermont as Vermont
from .virginia import Virginia as Virginia
from .washington import Washington as Washington
from .west_virginia import WestVirginia as WestVirginia
from .wisconsin import Wisconsin as Wisconsin
from .wyoming import Wyoming as Wyoming

__all__ = [
    "UnitedStates",  # Generic federal calendar
    "Alabama",
    "AlabamaBaldwinCounty",
    "AlabamaMobileCounty",
    "AlabamaPerryCounty",
    "Alaska",
    "Arizona",
    "Arkansas",
    "California",
    "CaliforniaEducation",
    "CaliforniaBerkeley",
    "CaliforniaSanFrancisco",
    "CaliforniaWestHollywood",
    "Colorado",
    "Connecticut",
    "Delaware",
    "DistrictOfColumbia",
    "Florida",
    "FloridaLegal",
    "FloridaCircuitCourts",
    "FloridaMiamiDade",
    "Georgia",
    "Hawaii",
    "Idaho",
    "Illinois",
    "ChicagoIllinois",
    "Indiana",
    "Iowa",
    "Kansas",
    "Kentucky",
    "Louisiana",
    "Maine",
    "Maryland",
    "Massachusetts",
    "SuffolkCountyMassachusetts",
    "Michigan",
    "Minnesota",
    "Mississippi",
    "Missouri",
    "Montana",
    "Nebraska",
    "Nevada",
    "NewHampshire",
    "NewJersey",
    "NewMexico",
    "NewYork",
    "NorthCarolina",
    "NorthDakota",
    "Ohio",
    "Oklahoma",
    "Oregon",
    "Pennsylvania",
    "RhodeIsland",
    "SouthCarolina",
    "SouthDakota",
    "Tennessee",
    "TexasBase",
    "Texas",
    "Utah",
    "Vermont",
    "Virginia",
    "Washington",
    "WestVirginia",
    "Wisconsin",
    "Wyoming",
    # Non-State territories
    "AmericanSamoa",
    "Guam",
    "FederalReserveSystem",
]



from _typeshed import Incomplete
from abc import ABC
from typing import Generic, TypeVar

_T_co = TypeVar("_T_co", covariant=True)

class Optional(ABC, Generic[_T_co]):
    def __getattr__(self, name: str) -> Incomplete: ...

def __getattr__(name: str): ...  # incomplete module



from typing import Final, Literal

EPOCH: Final = 347995.5
HEBREW_YEAR_OFFSET: Final = 3760
NISAN: Final = 1
IYYAR: Final = 2
SIVAN: Final = 3
TAMMUZ: Final = 4
AV: Final = 5
ELUL: Final = 6
TISHRI: Final = 7
HESHVAN: Final = 8
KISLEV: Final = 9
TEVETH: Final = 10
SHEVAT: Final = 11
ADAR: Final = 12
VEADAR: Final = 13
MONTHS: Final[list[str]]
MONTHS_HEB: Final[list[str]]

def leap(year: int) -> bool: ...
def year_months(year: int) -> Literal[12, 13]: ...
def delay_1(year: int) -> int: ...
def delay_2(year: int) -> Literal[0, 1, 2]: ...
def year_days(year: int) -> float: ...
def month_days(year: int, month: int) -> Literal[29, 30]: ...
def to_jd(year: int, month: int, day: int) -> float: ...
def from_jd(jd: float) -> tuple[int, int, int]: ...
def to_civil(year: int, month: int, day: int) -> tuple[int, int, int]: ...
def to_jd_gregorianyear(gregorianyear: int, hebrew_month: int, hebrew_day: int) -> float: ...
def from_gregorian(year: int, month: int, day: int) -> tuple[int, int, int]: ...
def to_gregorian(year: int, month: int, day: int) -> tuple[int, int, int]: ...
def monthcalendar(year: int, month: int) -> list[list[int | None]]: ...
def format(year: int, month: int, day: int, lang: str | None = None) -> str: ...



__all__ = ["spew", "unspew"]

from collections.abc import Container
from types import FrameType
from typing import Any
from typing_extensions import Self

class Spew:
    trace_names: Container[str] | None = None
    show_values: bool

    def __init__(self, trace_names: Container[str] | None = None, show_values: bool = True) -> None: ...
    def __call__(
        self, frame: FrameType, event: str, arg: Any  # `arg` is not used inside the function, stub is set Any
    ) -> Self: ...

def spew(trace_names: Container[str] | None = None, show_values: bool = False) -> None: ...
def unspew() -> None: ...



from typing import Final

from docutils import nodes
from docutils.parsers.rst import Directive

__docformat__: Final = "reStructuredText"

class BaseAdmonition(Directive):
    node_class: type[nodes.Admonition]  # Subclasses must set this to the appropriate admonition node class.

class Admonition(BaseAdmonition):
    node_class: type[nodes.admonition]

class Attention(BaseAdmonition):
    node_class: type[nodes.attention]

class Caution(BaseAdmonition):
    node_class: type[nodes.caution]

class Danger(BaseAdmonition):
    node_class: type[nodes.danger]

class Error(BaseAdmonition):
    node_class: type[nodes.error]

class Hint(BaseAdmonition):
    node_class: type[nodes.hint]

class Important(BaseAdmonition):
    node_class: type[nodes.important]

class Note(BaseAdmonition):
    node_class: type[nodes.note]

class Tip(BaseAdmonition):
    node_class: type[nodes.tip]

class Warning(BaseAdmonition):
    node_class: type[nodes.warning]



import builtins
import ctypes
import sys
from _typeshed import Incomplete
from types import TracebackType
from typing_extensions import Self

if sys.platform == "win32":
    def format_system_message(errno: int) -> str | None: ...

    class WindowsError(builtins.WindowsError):
        def __init__(self, value: int | None = None) -> None: ...
        @property
        def message(self) -> str: ...
        @property
        def code(self) -> int: ...

    def handle_nonzero_success(result: int) -> None: ...
    GMEM_MOVEABLE: int
    GlobalAlloc: Incomplete
    GlobalLock: Incomplete
    GlobalUnlock: Incomplete
    GlobalSize: Incomplete
    CreateFileMapping: Incomplete
    MapViewOfFile: Incomplete
    UnmapViewOfFile: Incomplete
    RtlMoveMemory: Incomplete

    class MemoryMap:
        name: str
        length: int
        security_attributes: Incomplete | None
        pos: int
        filemap: Incomplete
        view: Incomplete
        def __init__(self, name: str, length: int, security_attributes=None) -> None: ...
        def __enter__(self) -> Self: ...
        def seek(self, pos: int) -> None: ...
        def write(self, msg: bytes) -> None: ...
        def read(self, n: int) -> bytes: ...
        def __exit__(
            self, exc_type: type[BaseException] | None, exc_val: BaseException | None, tb: TracebackType | None
        ) -> None: ...

    READ_CONTROL: int
    STANDARD_RIGHTS_REQUIRED: int
    STANDARD_RIGHTS_READ: int
    STANDARD_RIGHTS_WRITE: int
    STANDARD_RIGHTS_EXECUTE: int
    STANDARD_RIGHTS_ALL: int
    POLICY_VIEW_LOCAL_INFORMATION: int
    POLICY_VIEW_AUDIT_INFORMATION: int
    POLICY_GET_PRIVATE_INFORMATION: int
    POLICY_TRUST_ADMIN: int
    POLICY_CREATE_ACCOUNT: int
    POLICY_CREATE_SECRET: int
    POLICY_CREATE_PRIVILEGE: int
    POLICY_SET_DEFAULT_QUOTA_LIMITS: int
    POLICY_SET_AUDIT_REQUIREMENTS: int
    POLICY_AUDIT_LOG_ADMIN: int
    POLICY_SERVER_ADMIN: int
    POLICY_LOOKUP_NAMES: int
    POLICY_NOTIFICATION: int
    POLICY_ALL_ACCESS: int
    POLICY_READ: int
    POLICY_WRITE: int
    POLICY_EXECUTE: int

    class TokenAccess:
        TOKEN_QUERY: int

    class TokenInformationClass:
        TokenUser: int

    class TOKEN_USER(ctypes.Structure):
        num: int
        SID: Incomplete
        ATTRIBUTES: Incomplete

    class SECURITY_DESCRIPTOR(ctypes.Structure):
        SECURITY_DESCRIPTOR_CONTROL: Incomplete
        REVISION: int
        Revision: int
        Sbz1: Incomplete
        Control: Incomplete
        Owner: Incomplete
        Group: Incomplete
        Sacl: Incomplete
        Dacl: Incomplete

    class SECURITY_ATTRIBUTES(ctypes.Structure):
        nLength: int
        lpSecurityDescriptor: int
        bInheritHandle: bool
        def __init__(self, *args, **kwargs) -> None: ...
        @property
        def descriptor(self): ...
        @descriptor.setter
        def descriptor(self, value) -> None: ...

    def GetTokenInformation(token, information_class): ...
    def OpenProcessToken(proc_handle, access): ...
    def get_current_user() -> TOKEN_USER: ...
    def get_security_attributes_for_user(user: TOKEN_USER | None = None) -> SECURITY_ATTRIBUTES: ...



from _typeshed import SupportsDunderGT, SupportsDunderLT, SupportsRead
from collections.abc import Iterable, Mapping, Sequence
from re import Pattern
from typing import Any, Final, Protocol, TypeVar, overload, type_check_only
from typing_extensions import TypeAlias

from Xlib.display import Display
from Xlib.support.lock import _DummyLock

_T = TypeVar("_T")
_T_contra = TypeVar("_T_contra", contravariant=True)

_DB: TypeAlias = dict[str, tuple[_DB, ...]]
# A recursive type can be a bit annoying due to dict invariance,
# so this is a slightly less precise version of the _DB alias for parameter annotations
_DB_Param: TypeAlias = dict[str, Any]

@type_check_only
class _SupportsComparisons(SupportsDunderLT[_T_contra], SupportsDunderGT[_T_contra], Protocol[_T_contra]): ...

comment_re: Final[Pattern[str]]
resource_spec_re: Final[Pattern[str]]
value_escape_re: Final[Pattern[str]]
resource_parts_re: Final[Pattern[str]]
NAME_MATCH: Final = 0
CLASS_MATCH: Final = 2
WILD_MATCH: Final = 4
MATCH_SKIP: Final = 6

class OptionError(Exception): ...

class ResourceDB:
    db: _DB
    lock: _DummyLock
    def __init__(
        self,
        file: bytes | SupportsRead[str] | None = None,
        string: str | None = None,
        resources: Iterable[tuple[str, object]] | None = None,
    ) -> None: ...
    def insert_file(self, file: bytes | SupportsRead[str]) -> None: ...
    def insert_string(self, data: str) -> None: ...
    def insert_resources(self, resources: Iterable[tuple[str, object]]) -> None: ...
    def insert(self, resource: str, value: object) -> None: ...
    def __getitem__(self, keys_tuple: tuple[str, str]) -> Any: ...
    @overload
    def get(self, res: str, cls: str, default: None = None) -> Any: ...
    @overload
    def get(self, res: str, cls: str, default: _T) -> _T: ...
    def update(self, db: ResourceDB) -> None: ...
    def output(self) -> str: ...
    def getopt(self, name: str, argv: Sequence[str], opts: Mapping[str, Option]) -> Sequence[str]: ...

def bin_insert(list: list[_SupportsComparisons[_T]], element: _SupportsComparisons[_T]) -> None: ...
def update_db(dest: _DB_Param, src: _DB_Param) -> None: ...
def copy_group(group: tuple[_DB_Param, ...]) -> tuple[_DB, ...]: ...
def copy_db(db: _DB_Param) -> _DB: ...
def output_db(prefix: str, db: _DB_Param) -> str: ...
def output_escape(value: object) -> str: ...

class Option:
    def parse(self, name: str, db: ResourceDB, args: Sequence[_T]) -> Sequence[_T]: ...

class NoArg(Option):
    specifier: str
    value: object
    def __init__(self, specifier: str, value: object) -> None: ...

class IsArg(Option):
    specifier: str
    def __init__(self, specifier: str) -> None: ...

class SepArg(Option):
    specifier: str
    def __init__(self, specifier: str) -> None: ...

class ResArgClass(Option):
    def parse(self, name: str, db: ResourceDB, args: Sequence[str]) -> Sequence[str]: ...  # type: ignore[override]

ResArg: ResArgClass

class SkipArgClass(Option): ...

SkipArg: SkipArgClass

class SkipLineClass(Option): ...

SkipLine: SkipLineClass

class SkipNArgs(Option):
    count: int
    def __init__(self, count: int) -> None: ...

def get_display_opts(
    options: Mapping[str, Option], argv: Sequence[str] = ...
) -> tuple[Display, str, ResourceDB, Sequence[str]]: ...

stdopts: Final[dict[str, SepArg | NoArg | ResArgClass]]



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import sys
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import tensorflow.core.framework.summary_pb2

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _WorkerHealth:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _WorkerHealthEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_WorkerHealth.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    OK: _WorkerHealth.ValueType  # 0
    """By default a worker is healthy."""
    RECEIVED_SHUTDOWN_SIGNAL: _WorkerHealth.ValueType  # 1
    INTERNAL_ERROR: _WorkerHealth.ValueType  # 2
    SHUTTING_DOWN: _WorkerHealth.ValueType  # 3
    """Worker has been instructed to shutdown after a timeout."""

class WorkerHealth(_WorkerHealth, metaclass=_WorkerHealthEnumTypeWrapper):
    """Worker heartbeat messages.  Support for these operations is currently
    internal and expected to change.

    Current health status of a worker.
    """

OK: WorkerHealth.ValueType  # 0
"""By default a worker is healthy."""
RECEIVED_SHUTDOWN_SIGNAL: WorkerHealth.ValueType  # 1
INTERNAL_ERROR: WorkerHealth.ValueType  # 2
SHUTTING_DOWN: WorkerHealth.ValueType  # 3
"""Worker has been instructed to shutdown after a timeout."""
global___WorkerHealth = WorkerHealth

class _WorkerShutdownMode:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _WorkerShutdownModeEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_WorkerShutdownMode.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    DEFAULT: _WorkerShutdownMode.ValueType  # 0
    NOT_CONFIGURED: _WorkerShutdownMode.ValueType  # 1
    WAIT_FOR_COORDINATOR: _WorkerShutdownMode.ValueType  # 2
    SHUTDOWN_AFTER_TIMEOUT: _WorkerShutdownMode.ValueType  # 3

class WorkerShutdownMode(_WorkerShutdownMode, metaclass=_WorkerShutdownModeEnumTypeWrapper):
    """Indicates the behavior of the worker when an internal error or shutdown
    signal is received.
    """

DEFAULT: WorkerShutdownMode.ValueType  # 0
NOT_CONFIGURED: WorkerShutdownMode.ValueType  # 1
WAIT_FOR_COORDINATOR: WorkerShutdownMode.ValueType  # 2
SHUTDOWN_AFTER_TIMEOUT: WorkerShutdownMode.ValueType  # 3
global___WorkerShutdownMode = WorkerShutdownMode

@typing.final
class Event(google.protobuf.message.Message):
    """Protocol buffer representing an event that happened during
    the execution of a Brain model.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    WALL_TIME_FIELD_NUMBER: builtins.int
    STEP_FIELD_NUMBER: builtins.int
    FILE_VERSION_FIELD_NUMBER: builtins.int
    GRAPH_DEF_FIELD_NUMBER: builtins.int
    SUMMARY_FIELD_NUMBER: builtins.int
    LOG_MESSAGE_FIELD_NUMBER: builtins.int
    SESSION_LOG_FIELD_NUMBER: builtins.int
    TAGGED_RUN_METADATA_FIELD_NUMBER: builtins.int
    META_GRAPH_DEF_FIELD_NUMBER: builtins.int
    SOURCE_METADATA_FIELD_NUMBER: builtins.int
    wall_time: builtins.float
    """Timestamp of the event."""
    step: builtins.int
    """Global step of the event."""
    file_version: builtins.str
    """An event file was started, with the specified version.
    This is use to identify the contents of the record IO files
    easily.  Current version is "brain.Event:2".  All versions
    start with "brain.Event:".
    """
    graph_def: builtins.bytes
    """An encoded version of a GraphDef."""
    meta_graph_def: builtins.bytes
    """An encoded version of a MetaGraphDef."""
    @property
    def summary(self) -> tensorflow.core.framework.summary_pb2.Summary:
        """A summary was generated."""

    @property
    def log_message(self) -> global___LogMessage:
        """The user output a log message. This was theoretically used by the defunct
        tensorboard_logging module, which has since been removed; this field is
        now deprecated and should not be used.
        """

    @property
    def session_log(self) -> global___SessionLog:
        """The state of the session which can be used for restarting after crashes."""

    @property
    def tagged_run_metadata(self) -> global___TaggedRunMetadata:
        """The metadata returned by running a session.run() call."""

    @property
    def source_metadata(self) -> global___SourceMetadata:
        """Information of the source that writes the events, this is only logged in
        the very first event along with the `file_version` field.
        """

    def __init__(
        self,
        *,
        wall_time: builtins.float | None = ...,
        step: builtins.int | None = ...,
        file_version: builtins.str | None = ...,
        graph_def: builtins.bytes | None = ...,
        summary: tensorflow.core.framework.summary_pb2.Summary | None = ...,
        log_message: global___LogMessage | None = ...,
        session_log: global___SessionLog | None = ...,
        tagged_run_metadata: global___TaggedRunMetadata | None = ...,
        meta_graph_def: builtins.bytes | None = ...,
        source_metadata: global___SourceMetadata | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "file_version",
            b"file_version",
            "graph_def",
            b"graph_def",
            "log_message",
            b"log_message",
            "meta_graph_def",
            b"meta_graph_def",
            "session_log",
            b"session_log",
            "source_metadata",
            b"source_metadata",
            "summary",
            b"summary",
            "tagged_run_metadata",
            b"tagged_run_metadata",
            "what",
            b"what",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "file_version",
            b"file_version",
            "graph_def",
            b"graph_def",
            "log_message",
            b"log_message",
            "meta_graph_def",
            b"meta_graph_def",
            "session_log",
            b"session_log",
            "source_metadata",
            b"source_metadata",
            "step",
            b"step",
            "summary",
            b"summary",
            "tagged_run_metadata",
            b"tagged_run_metadata",
            "wall_time",
            b"wall_time",
            "what",
            b"what",
        ],
    ) -> None: ...
    def WhichOneof(
        self, oneof_group: typing.Literal["what", b"what"]
    ) -> (
        typing.Literal[
            "file_version", "graph_def", "summary", "log_message", "session_log", "tagged_run_metadata", "meta_graph_def"
        ]
        | None
    ): ...

global___Event = Event

@typing.final
class SourceMetadata(google.protobuf.message.Message):
    """Holds the information of the source that writes the events."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    WRITER_FIELD_NUMBER: builtins.int
    writer: builtins.str
    """Low level name of the summary writer, such as
    `tensorflow.core.util.events_writer`.
    """
    def __init__(self, *, writer: builtins.str | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["writer", b"writer"]) -> None: ...

global___SourceMetadata = SourceMetadata

@typing.final
class LogMessage(google.protobuf.message.Message):
    """Protocol buffer used for logging messages to the events file.

    This was theoretically used by the defunct tensorboard_logging module, which
    has been removed; this message is now deprecated and should not be used.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Level:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _LevelEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[LogMessage._Level.ValueType], builtins.type
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        UNKNOWN: LogMessage._Level.ValueType  # 0
        DEBUGGING: LogMessage._Level.ValueType  # 10
        """Note: The logging level 10 cannot be named DEBUG. Some software
        projects compile their C/C++ code with -DDEBUG in debug builds. So the
        C++ code generated from this file should not have an identifier named
        DEBUG.
        """
        INFO: LogMessage._Level.ValueType  # 20
        WARN: LogMessage._Level.ValueType  # 30
        ERROR: LogMessage._Level.ValueType  # 40
        FATAL: LogMessage._Level.ValueType  # 50

    class Level(_Level, metaclass=_LevelEnumTypeWrapper): ...
    UNKNOWN: LogMessage.Level.ValueType  # 0
    DEBUGGING: LogMessage.Level.ValueType  # 10
    """Note: The logging level 10 cannot be named DEBUG. Some software
    projects compile their C/C++ code with -DDEBUG in debug builds. So the
    C++ code generated from this file should not have an identifier named
    DEBUG.
    """
    INFO: LogMessage.Level.ValueType  # 20
    WARN: LogMessage.Level.ValueType  # 30
    ERROR: LogMessage.Level.ValueType  # 40
    FATAL: LogMessage.Level.ValueType  # 50

    LEVEL_FIELD_NUMBER: builtins.int
    MESSAGE_FIELD_NUMBER: builtins.int
    level: global___LogMessage.Level.ValueType
    message: builtins.str
    def __init__(
        self, *, level: global___LogMessage.Level.ValueType | None = ..., message: builtins.str | None = ...
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["level", b"level", "message", b"message"]) -> None: ...

global___LogMessage = LogMessage

@typing.final
class SessionLog(google.protobuf.message.Message):
    """Protocol buffer used for logging session state."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _SessionStatus:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _SessionStatusEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[SessionLog._SessionStatus.ValueType], builtins.type
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        STATUS_UNSPECIFIED: SessionLog._SessionStatus.ValueType  # 0
        START: SessionLog._SessionStatus.ValueType  # 1
        STOP: SessionLog._SessionStatus.ValueType  # 2
        CHECKPOINT: SessionLog._SessionStatus.ValueType  # 3

    class SessionStatus(_SessionStatus, metaclass=_SessionStatusEnumTypeWrapper): ...
    STATUS_UNSPECIFIED: SessionLog.SessionStatus.ValueType  # 0
    START: SessionLog.SessionStatus.ValueType  # 1
    STOP: SessionLog.SessionStatus.ValueType  # 2
    CHECKPOINT: SessionLog.SessionStatus.ValueType  # 3

    STATUS_FIELD_NUMBER: builtins.int
    CHECKPOINT_PATH_FIELD_NUMBER: builtins.int
    MSG_FIELD_NUMBER: builtins.int
    status: global___SessionLog.SessionStatus.ValueType
    checkpoint_path: builtins.str
    """This checkpoint_path contains both the path and filename."""
    msg: builtins.str
    def __init__(
        self,
        *,
        status: global___SessionLog.SessionStatus.ValueType | None = ...,
        checkpoint_path: builtins.str | None = ...,
        msg: builtins.str | None = ...,
    ) -> None: ...
    def ClearField(
        self, field_name: typing.Literal["checkpoint_path", b"checkpoint_path", "msg", b"msg", "status", b"status"]
    ) -> None: ...

global___SessionLog = SessionLog

@typing.final
class TaggedRunMetadata(google.protobuf.message.Message):
    """For logging the metadata output for a single session.run() call."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TAG_FIELD_NUMBER: builtins.int
    RUN_METADATA_FIELD_NUMBER: builtins.int
    tag: builtins.str
    """Tag name associated with this metadata."""
    run_metadata: builtins.bytes
    """Byte-encoded version of the `RunMetadata` proto in order to allow lazy
    deserialization.
    """
    def __init__(self, *, tag: builtins.str | None = ..., run_metadata: builtins.bytes | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["run_metadata", b"run_metadata", "tag", b"tag"]) -> None: ...

global___TaggedRunMetadata = TaggedRunMetadata

@typing.final
class WatchdogConfig(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TIMEOUT_MS_FIELD_NUMBER: builtins.int
    timeout_ms: builtins.int
    def __init__(self, *, timeout_ms: builtins.int | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["timeout_ms", b"timeout_ms"]) -> None: ...

global___WatchdogConfig = WatchdogConfig

@typing.final
class RequestedExitCode(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    EXIT_CODE_FIELD_NUMBER: builtins.int
    exit_code: builtins.int
    def __init__(self, *, exit_code: builtins.int | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["exit_code", b"exit_code"]) -> None: ...

global___RequestedExitCode = RequestedExitCode

@typing.final
class WorkerHeartbeatRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SHUTDOWN_MODE_FIELD_NUMBER: builtins.int
    WATCHDOG_CONFIG_FIELD_NUMBER: builtins.int
    EXIT_CODE_FIELD_NUMBER: builtins.int
    shutdown_mode: global___WorkerShutdownMode.ValueType
    @property
    def watchdog_config(self) -> global___WatchdogConfig: ...
    @property
    def exit_code(self) -> global___RequestedExitCode: ...
    def __init__(
        self,
        *,
        shutdown_mode: global___WorkerShutdownMode.ValueType | None = ...,
        watchdog_config: global___WatchdogConfig | None = ...,
        exit_code: global___RequestedExitCode | None = ...,
    ) -> None: ...
    def HasField(
        self, field_name: typing.Literal["exit_code", b"exit_code", "watchdog_config", b"watchdog_config"]
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "exit_code", b"exit_code", "shutdown_mode", b"shutdown_mode", "watchdog_config", b"watchdog_config"
        ],
    ) -> None: ...

global___WorkerHeartbeatRequest = WorkerHeartbeatRequest

@typing.final
class WorkerHeartbeatResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HEALTH_STATUS_FIELD_NUMBER: builtins.int
    WORKER_LOG_FIELD_NUMBER: builtins.int
    HOSTNAME_FIELD_NUMBER: builtins.int
    health_status: global___WorkerHealth.ValueType
    hostname: builtins.str
    @property
    def worker_log(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Event]: ...
    def __init__(
        self,
        *,
        health_status: global___WorkerHealth.ValueType | None = ...,
        worker_log: collections.abc.Iterable[global___Event] | None = ...,
        hostname: builtins.str | None = ...,
    ) -> None: ...
    def ClearField(
        self, field_name: typing.Literal["health_status", b"health_status", "hostname", b"hostname", "worker_log", b"worker_log"]
    ) -> None: ...

global___WorkerHeartbeatResponse = WorkerHeartbeatResponse



from authlib.oauth1.rfc5849.base_server import BaseServer

class ResourceProtector(BaseServer):
    def validate_request(self, method, uri, body, headers): ...
    def get_token_credential(self, request) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import tensorflow.core.framework.tensor_pb2
import tensorflow.core.framework.tensor_shape_pb2
import tensorflow.core.framework.types_pb2

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class AttrValue(google.protobuf.message.Message):
    """Protocol buffer representing the value for an attr used to configure an Op.
    Comment indicates the corresponding attr type.  Only the field matching the
    attr type may be filled.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class ListValue(google.protobuf.message.Message):
        """LINT.IfChange"""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        S_FIELD_NUMBER: builtins.int
        I_FIELD_NUMBER: builtins.int
        F_FIELD_NUMBER: builtins.int
        B_FIELD_NUMBER: builtins.int
        TYPE_FIELD_NUMBER: builtins.int
        SHAPE_FIELD_NUMBER: builtins.int
        TENSOR_FIELD_NUMBER: builtins.int
        FUNC_FIELD_NUMBER: builtins.int
        @property
        def s(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.bytes]:
            """ "list(string)" """

        @property
        def i(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
            """ "list(int)" """

        @property
        def f(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]:
            """ "list(float)" """

        @property
        def b(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.bool]:
            """ "list(bool)" """

        @property
        def type(
            self,
        ) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[
            tensorflow.core.framework.types_pb2.DataType.ValueType
        ]:
            """ "list(type)" """

        @property
        def shape(
            self,
        ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
            tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto
        ]:
            """ "list(shape)" """

        @property
        def tensor(
            self,
        ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
            tensorflow.core.framework.tensor_pb2.TensorProto
        ]:
            """ "list(tensor)" """

        @property
        def func(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NameAttrList]:
            """ "list(attr)" """

        def __init__(
            self,
            *,
            s: collections.abc.Iterable[builtins.bytes] | None = ...,
            i: collections.abc.Iterable[builtins.int] | None = ...,
            f: collections.abc.Iterable[builtins.float] | None = ...,
            b: collections.abc.Iterable[builtins.bool] | None = ...,
            type: collections.abc.Iterable[tensorflow.core.framework.types_pb2.DataType.ValueType] | None = ...,
            shape: collections.abc.Iterable[tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto] | None = ...,
            tensor: collections.abc.Iterable[tensorflow.core.framework.tensor_pb2.TensorProto] | None = ...,
            func: collections.abc.Iterable[global___NameAttrList] | None = ...,
        ) -> None: ...
        def ClearField(
            self,
            field_name: typing.Literal[
                "b",
                b"b",
                "f",
                b"f",
                "func",
                b"func",
                "i",
                b"i",
                "s",
                b"s",
                "shape",
                b"shape",
                "tensor",
                b"tensor",
                "type",
                b"type",
            ],
        ) -> None: ...

    S_FIELD_NUMBER: builtins.int
    I_FIELD_NUMBER: builtins.int
    F_FIELD_NUMBER: builtins.int
    B_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    SHAPE_FIELD_NUMBER: builtins.int
    TENSOR_FIELD_NUMBER: builtins.int
    LIST_FIELD_NUMBER: builtins.int
    FUNC_FIELD_NUMBER: builtins.int
    PLACEHOLDER_FIELD_NUMBER: builtins.int
    s: builtins.bytes
    """"string" """
    i: builtins.int
    """"int" """
    f: builtins.float
    """"float" """
    b: builtins.bool
    """"bool" """
    type: tensorflow.core.framework.types_pb2.DataType.ValueType
    """"type" """
    placeholder: builtins.str
    """This is a placeholder only used in nodes defined inside a
    function.  It indicates the attr value will be supplied when
    the function is instantiated.  For example, let us suppose a
    node "N" in function "FN". "N" has an attr "A" with value
    placeholder = "foo". When FN is instantiated with attr "foo"
    set to "bar", the instantiated node N's attr A will have been
    given the value "bar".
    """
    @property
    def shape(self) -> tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto:
        """ "shape" """

    @property
    def tensor(self) -> tensorflow.core.framework.tensor_pb2.TensorProto:
        """ "tensor" """

    @property
    def list(self) -> global___AttrValue.ListValue:
        """any "list(...)" """

    @property
    def func(self) -> global___NameAttrList:
        """ "func" represents a function. func.name is a function's name or
        a primitive op's name. func.attr.first is the name of an attr
        defined for that function. func.attr.second is the value for
        that attr in the instantiation.
        """

    def __init__(
        self,
        *,
        s: builtins.bytes | None = ...,
        i: builtins.int | None = ...,
        f: builtins.float | None = ...,
        b: builtins.bool | None = ...,
        type: tensorflow.core.framework.types_pb2.DataType.ValueType | None = ...,
        shape: tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto | None = ...,
        tensor: tensorflow.core.framework.tensor_pb2.TensorProto | None = ...,
        list: global___AttrValue.ListValue | None = ...,
        func: global___NameAttrList | None = ...,
        placeholder: builtins.str | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "b",
            b"b",
            "f",
            b"f",
            "func",
            b"func",
            "i",
            b"i",
            "list",
            b"list",
            "placeholder",
            b"placeholder",
            "s",
            b"s",
            "shape",
            b"shape",
            "tensor",
            b"tensor",
            "type",
            b"type",
            "value",
            b"value",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "b",
            b"b",
            "f",
            b"f",
            "func",
            b"func",
            "i",
            b"i",
            "list",
            b"list",
            "placeholder",
            b"placeholder",
            "s",
            b"s",
            "shape",
            b"shape",
            "tensor",
            b"tensor",
            "type",
            b"type",
            "value",
            b"value",
        ],
    ) -> None: ...
    def WhichOneof(
        self, oneof_group: typing.Literal["value", b"value"]
    ) -> typing.Literal["s", "i", "f", "b", "type", "shape", "tensor", "list", "func", "placeholder"] | None: ...

global___AttrValue = AttrValue

@typing.final
class NameAttrList(google.protobuf.message.Message):
    """A list of attr names and their values. The whole list is attached
    with a string name.  E.g., MatMul[T=float].
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class AttrEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___AttrValue: ...
        def __init__(self, *, key: builtins.str | None = ..., value: global___AttrValue | None = ...) -> None: ...
        def HasField(self, field_name: typing.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    NAME_FIELD_NUMBER: builtins.int
    ATTR_FIELD_NUMBER: builtins.int
    name: builtins.str
    @property
    def attr(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___AttrValue]: ...
    def __init__(
        self, *, name: builtins.str | None = ..., attr: collections.abc.Mapping[builtins.str, global___AttrValue] | None = ...
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["attr", b"attr", "name", b"name"]) -> None: ...

global___NameAttrList = NameAttrList



from typing import Any

from ..strategy.base import BaseStrategy

LDAP_MESSAGE_TEMPLATE: Any

class SyncStrategy(BaseStrategy):
    sync: bool
    no_real_dsa: bool
    pooled: bool
    can_stream: bool
    socket_size: Any
    def __init__(self, ldap_connection) -> None: ...
    def open(self, reset_usage: bool = True, read_server_info: bool = True) -> None: ...
    def receiving(self): ...
    def post_send_single_response(self, message_id): ...
    def post_send_search(self, message_id): ...
    def set_stream(self, value) -> None: ...
    def get_stream(self) -> None: ...



from authlib.oauth2 import OAuth2Error

class AuthorizationPendingError(OAuth2Error):
    error: str

class SlowDownError(OAuth2Error):
    error: str

class ExpiredTokenError(OAuth2Error):
    error: str



from _typeshed import Incomplete
from typing import Final

from auth0.rest import RestClient
from auth0.types import RequestData

UNKNOWN_ERROR: Final[str]

class AuthenticationBase:
    domain: str
    client_id: str
    client_secret: str | None
    client_assertion_signing_key: str | None
    client_assertion_signing_alg: str | None
    protocol: str
    client: RestClient
    def __init__(
        self,
        domain: str,
        client_id: str,
        client_secret: str | None = None,
        client_assertion_signing_key: str | None = None,
        client_assertion_signing_alg: str | None = None,
        telemetry: bool = True,
        timeout: float | tuple[float, float] = 5.0,
        protocol: str = "https",
    ) -> None: ...
    def post(self, url: str, data: RequestData | None = None, headers: dict[str, str] | None = None): ...
    def authenticated_post(self, url: str, data: dict[str, Incomplete], headers: dict[str, str] | None = None): ...
    def get(self, url: str, params: dict[str, Incomplete] | None = None, headers: dict[str, str] | None = None): ...



from _typeshed import Incomplete

class OAuth1Request:
    method: Incomplete
    uri: Incomplete
    body: Incomplete
    headers: Incomplete
    client: Incomplete
    credential: Incomplete
    user: Incomplete
    query: Incomplete
    query_params: Incomplete
    body_params: Incomplete
    params: Incomplete
    def __init__(self, method, uri, body=None, headers=None) -> None: ...
    @property
    def client_id(self): ...
    @property
    def client_secret(self): ...
    @property
    def rsa_public_key(self): ...
    @property
    def timestamp(self): ...
    @property
    def redirect_uri(self): ...
    @property
    def signature(self): ...
    @property
    def signature_method(self): ...
    @property
    def token(self): ...
    @property
    def token_secret(self): ...



from typing import Any, Literal

from .YoutubeDL import YoutubeDL

class Cache:
    def __init__(self, ydl: YoutubeDL) -> None: ...
    @property
    def enabled(self) -> bool: ...
    def store(
        self, section: str, key: str, data: Any, dtype: Literal["json"] = "json"  # data is anything JSON serializable.
    ) -> None: ...
    def load(
        self,
        section: str,
        key: str,
        dtype: Literal["json"] = "json",
        default: Any = None,  # Returned if not enabled or if the cache entry is not found.
        *,
        min_ver: str | None = None,
    ) -> Any: ...  # Anything JSON serializable.
    def remove(self) -> None: ...



from _typeshed import FileDescriptor, StrOrBytesPath
from collections.abc import Callable
from types import TracebackType
from typing import Any, Literal, overload
from typing_extensions import Self, TypeVarTuple, Unpack

from gevent._types import _Loop, _StatResult

_Ts = TypeVarTuple("_Ts")

class AbstractWatcherType(type):
    def new_handle(cls, obj: object) -> int: ...
    def new(cls, kind: object) -> Any: ...

class watcher(metaclass=AbstractWatcherType):
    loop: _Loop
    def __init__(self, _loop: _Loop, ref: bool = True, priority: int | None = None, args: tuple[object, ...] = ()) -> None: ...
    def close(self) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, t: type[BaseException] | None, v: BaseException | None, tb: TracebackType | None) -> None: ...
    @property
    def ref(self) -> bool: ...
    callback: Callable[..., Any]
    args: tuple[Any, ...]
    def start(self, callback: Callable[[Unpack[_Ts]], Any], *args: Unpack[_Ts]) -> None: ...
    def stop(self) -> None: ...
    @property
    def priority(self) -> int | None: ...
    @priority.setter
    def priority(self, value: int | None) -> None: ...
    @property
    def active(self) -> bool: ...
    @property
    def pending(self) -> bool: ...

class IoMixin:
    EVENT_MASK: int
    def __init__(self, loop: _Loop, fd: FileDescriptor, events: int, ref: bool = True, priority: int | None = None) -> None: ...
    @overload
    def start(self, callback: Callable[[int, Unpack[_Ts]], Any], *args: Unpack[_Ts], pass_events: Literal[True]) -> None: ...
    @overload
    def start(self, callback: Callable[[Unpack[_Ts]], Any], *args: Unpack[_Ts]) -> None: ...

class TimerMixin:
    def __init__(
        self, loop: _Loop, after: float = 0.0, repeat: float = 0.0, ref: bool = True, priority: int | None = None
    ) -> None: ...
    @overload
    def start(self, callback: Callable[[Unpack[_Ts]], object], *args: Unpack[_Ts], update: bool) -> None: ...
    @overload
    def start(self, callback: Callable[[Unpack[_Ts]], object], *args: Unpack[_Ts]) -> None: ...
    @overload
    def again(self, callback: Callable[[Unpack[_Ts]], object], *args: Unpack[_Ts], update: bool) -> None: ...
    @overload
    def again(self, callback: Callable[[Unpack[_Ts]], object], *args: Unpack[_Ts]) -> None: ...

class SignalMixin:
    def __init__(self, loop: _Loop, signalnum: int, ref: bool = True, priority: int | None = None) -> None: ...

class IdleMixin: ...
class PrepareMixin: ...
class CheckMixin: ...
class ForkMixin: ...

class AsyncMixin:
    def send(self) -> None: ...
    def send_ignoring_arg(self, _ignored: object) -> None: ...
    @property
    def pending(self) -> bool: ...

class ChildMixin:
    def __init__(self, loop: _Loop, pid: int, trace: int = 0, ref: bool = True) -> None: ...
    @property
    def pid(self) -> int: ...
    @property
    def rpid(self) -> int | None: ...
    @property
    def rstatus(self) -> int: ...

class StatMixin:
    def __init__(
        self, _loop: _Loop, path: StrOrBytesPath, interval: float = 0.0, ref: bool = True, priority: float | None = None
    ) -> None: ...
    @property
    def path(self) -> StrOrBytesPath: ...
    @property
    def attr(self) -> _StatResult | None: ...
    @property
    def prev(self) -> _StatResult | None: ...
    @property
    def interval(self) -> float: ...

__all__: list[str] = []



from _typeshed import Incomplete, MaybeNone
from typing import ClassVar

from ..cmd import Command

class install_lib(Command):
    description: ClassVar[str]
    user_options: ClassVar[list[tuple[str, str | None, str]]]
    boolean_options: ClassVar[list[str]]
    negative_opt: ClassVar[dict[str, str]]
    install_dir: Incomplete
    build_dir: Incomplete
    force: bool
    compile: Incomplete
    optimize: Incomplete
    skip_build: Incomplete
    def initialize_options(self) -> None: ...
    def finalize_options(self) -> None: ...
    def run(self) -> None: ...
    def build(self) -> None: ...
    def install(self) -> list[str] | MaybeNone: ...
    def byte_compile(self, files) -> None: ...
    def get_outputs(self): ...
    def get_inputs(self): ...



from _typeshed import Incomplete
from collections.abc import Mapping
from typing import Any, Literal
from typing_extensions import Self

from jwcrypto.common import JWException, JWSEHeaderParameter
from jwcrypto.jwa import JWAAlgorithm
from jwcrypto.jwk import JWK, JWKSet

JWSHeaderRegistry: Mapping[str, JWSEHeaderParameter]
default_allowed_algs: list[str]

class InvalidJWSSignature(JWException):
    def __init__(self, message: str | None = None, exception: BaseException | None = None) -> None: ...

class InvalidJWSObject(JWException):
    def __init__(self, message: str | None = None, exception: BaseException | None = None) -> None: ...

class InvalidJWSOperation(JWException):
    def __init__(self, message: str | None = None, exception: BaseException | None = None) -> None: ...

class JWSCore:
    alg: str
    engine: JWAAlgorithm
    key: JWK | JWKSet
    header: dict[str, Any]
    protected: str
    payload: bytes
    def __init__(
        self,
        alg: str,
        key: JWK | JWKSet,
        header: dict[str, Any] | str | None,
        payload: str | bytes,
        algs: list[str] | None = None,
    ) -> None: ...
    def sign(self) -> dict[str, str | bytes]: ...
    def verify(self, signature: bytes) -> Literal[True]: ...

class JWS:
    objects: Incomplete
    verifylog: list[str] | None
    header_registry: Incomplete
    def __init__(self, payload=None, header_registry=None) -> None: ...
    @property
    def allowed_algs(self): ...
    @allowed_algs.setter
    def allowed_algs(self, algs) -> None: ...
    @property
    def is_valid(self): ...
    def verify(self, key, alg=None, detached_payload=None) -> None: ...
    def deserialize(self, raw_jws, key=None, alg=None) -> None: ...
    def add_signature(self, key, alg=None, protected=None, header=None) -> None: ...
    def serialize(self, compact: bool = False) -> str: ...
    @property
    def payload(self): ...
    def detach_payload(self) -> None: ...
    @property
    def jose_header(self): ...
    @classmethod
    def from_jose_token(cls, token: str | bytes) -> Self: ...
    def __eq__(self, other: object) -> bool: ...



import ast
from collections.abc import Callable, Iterable
from types import FrameType
from typing import Any, Literal, TypeVar
from typing_extensions import TypeAlias

from .watch_print import WatchPrint

_T = TypeVar("_T")
_TrackKind: TypeAlias = Literal["object", "variable"] | list[Literal["object", "variable"]]

class WatchElement:
    # User-defined callbacks passed to `__init__` set as instance variables have arguments of type `Any` to be
    # compatible with more precisely-annotated signatures. These callbacks are passed from `watchpoints.watch.Watch`.

    alias: str | None
    attr: str | None
    cmp: Callable[[Any, Any], bool] | None
    copy: Callable[[Any], object] | None  # User-defined copy callback
    default_alias: str | None
    deepcopy: bool
    exist: bool
    frame: FrameType
    localvar: str | None
    obj: Any
    parent: Any
    prev_obj: Any
    prev_obj_repr: str
    subscr: Any
    watch_print: WatchPrint
    when: Callable[[Any], bool] | None

    def __init__(
        self,
        frame: FrameType,
        node: ast.expr,
        *,
        alias: str | None = ...,
        callback: Callable[[FrameType, WatchElement, tuple[str, str, int | None]], None] | None = ...,
        cmp: Callable[[Any, Any], bool] | None = ...,  # User-defined comparison callback
        copy: Callable[[_T], _T] | None = ...,
        deepcopy: bool = False,
        default_alias: str | None = ...,
        track: _TrackKind = ...,
        watch_print: WatchPrint = ...,
        when: Callable[[Any], bool] | None = ...,  # User-defined callback for conditional watchpoints
    ) -> None: ...
    def belong_to(self, lst: Iterable[object]) -> bool: ...
    def changed(self, frame: FrameType) -> tuple[bool, bool]: ...
    def obj_changed(self, other: object) -> bool: ...
    def same(self, other: object) -> bool: ...
    @property
    def track(self) -> _TrackKind: ...
    @track.setter
    def track(self, val: _TrackKind) -> None: ...
    def update(self) -> None: ...



import grpc
from google.protobuf.descriptor_database import DescriptorDatabase
from google.protobuf.descriptor_pb2 import FileDescriptorProto

class ProtoReflectionDescriptorDatabase(DescriptorDatabase):
    def __init__(self, channel: grpc.Channel) -> None: ...
    def get_services(self) -> list[str]: ...
    def FindFileByName(self, name: str) -> FileDescriptorProto: ...
    def FindFileContainingSymbol(self, symbol: str) -> FileDescriptorProto: ...
    def FindAllExtensionNumbers(self, extendee_name: str) -> list[int]: ...
    def FindFileContainingExtension(self, extendee_name: str, extension_number: int) -> FileDescriptorProto: ...



import threading
from _typeshed import Incomplete
from typing import Final, Literal, type_check_only

# Not bothering with types here as lxml support is supposed to be dropped in a future version of defusedxml

LXML3: bool
__origin__: Final = "lxml.etree"

def tostring(
    element_or_tree,
    *,
    encoding: str | None = None,
    method: Literal["xml", "html", "text", "c14n", "c14n2"] = "xml",
    xml_declaration: bool | None = None,
    pretty_print: bool = False,
    with_tail: bool = True,
    standalone: bool | None = None,
    doctype=None,
    exclusive: bool = False,
    inclusive_ns_prefixes=None,
    with_comments: bool = True,
    strip_text: bool = False,
): ...

# Should be imported from lxml.etree.ElementBase, but lxml lacks types
@type_check_only
class _ElementBase: ...

class RestrictedElement(_ElementBase):
    __slots__ = ()
    blacklist: Incomplete
    def __iter__(self): ...
    def iterchildren(self, tag=None, reversed: bool = False): ...
    def iter(self, tag=None, *tags): ...
    def iterdescendants(self, tag=None, *tags): ...
    def itersiblings(self, tag=None, preceding: bool = False): ...
    def getchildren(self): ...
    def getiterator(self, tag=None): ...

class GlobalParserTLS(threading.local):
    parser_config: Incomplete
    element_class: Incomplete
    def createDefaultParser(self): ...
    def setDefaultParser(self, parser) -> None: ...
    def getDefaultParser(self): ...

def getDefaultParser(): ...
def check_docinfo(elementtree, forbid_dtd: bool = False, forbid_entities: bool = True) -> None: ...
def parse(source, parser=None, base_url=None, forbid_dtd: bool = False, forbid_entities: bool = True): ...
def fromstring(text, parser=None, base_url=None, forbid_dtd: bool = False, forbid_entities: bool = True): ...

XML = fromstring

def iterparse(*args, **kwargs) -> None: ...



from _typeshed import SupportsKeysAndGetItem
from collections.abc import Generator, ItemsView, Iterable, KeysView, ValuesView
from typing import NoReturn, TypeVar, overload
from typing_extensions import Self, TypeAlias

_KT = TypeVar("_KT")
_VT = TypeVar("_VT")
_T = TypeVar("_T")

class OrderedMultiDict(dict[_KT, _VT]):
    def add(self, k: _KT, v: _VT) -> None: ...
    def addlist(self, k: _KT, v: Iterable[_VT]) -> None: ...
    def clear(self) -> None: ...
    def copy(self) -> Self: ...
    def counts(self) -> Self: ...
    @classmethod
    def fromkeys(cls, keys: _KT, default: _VT | None = None) -> Self: ...  # type: ignore[override]
    @overload  # type: ignore[override]
    def get(self, k: _KT, default: None = None) -> _VT | None: ...
    @overload
    def get(self, k: _KT, default: _VT) -> _VT: ...
    def getlist(self, k: _KT, default: list[_VT] = ...) -> list[_VT]: ...
    def inverted(self) -> Self: ...
    def items(self, multi: bool = False) -> list[tuple[_KT, _VT]]: ...  # type: ignore[override]
    def iteritems(self, multi: bool = False) -> Generator[tuple[_KT, _VT], None, None]: ...
    def iterkeys(self, multi: bool = False) -> Generator[_KT, None, None]: ...
    def itervalues(self, multi: bool = False) -> Generator[_VT, None, None]: ...
    def keys(self, multi: bool = False) -> list[_KT]: ...  # type: ignore[override]
    def pop(self, k: _KT, default: _VT = ...) -> _VT: ...  # type: ignore[override]
    def popall(self, k: _KT, default: _VT = ...) -> list[_VT]: ...
    def poplast(self, k: _KT = ..., default: _VT = ...) -> _VT: ...
    @overload  # type: ignore[override]
    def setdefault(self, k: _KT, default: None = None) -> _VT | None: ...
    @overload
    def setdefault(self, k: _KT, default: _VT) -> _VT: ...
    def sorted(self, key: _KT | None = None, reverse: bool = False) -> Self: ...
    def sortedvalues(self, key: _KT | None = None, reverse: bool = False) -> Self: ...
    def todict(self, multi: bool = False) -> dict[_KT, _VT]: ...
    def update(self, E: SupportsKeysAndGetItem[_KT, _VT] | Iterable[tuple[_KT, _VT]], **F) -> None: ...  # type: ignore[override]
    def update_extend(self, E: SupportsKeysAndGetItem[_KT, _VT] | Iterable[tuple[_KT, _VT]], **F) -> None: ...
    def values(self, multi: bool = False) -> list[_VT]: ...  # type: ignore[override]
    def viewitems(self) -> ItemsView[_KT, _VT]: ...
    def viewkeys(self) -> KeysView[_KT]: ...
    def viewvalues(self) -> ValuesView[_VT]: ...

OMD: TypeAlias = OrderedMultiDict[_KT, _VT]
MultiDict: TypeAlias = OrderedMultiDict[_KT, _VT]

class FastIterOrderedMultiDict(OrderedMultiDict[_KT, _VT]):  # undocumented
    def iteritems(self, multi: bool = False) -> Generator[tuple[_KT, _VT], None, None]: ...
    def iterkeys(self, multi: bool = False) -> Generator[_KT, None, None]: ...

class OneToOne(dict[_KT, _VT]):
    __slots__ = ("inv",)
    inv: OneToOne[_VT, _KT]
    def clear(self) -> None: ...
    def copy(self) -> Self: ...
    def pop(self, key: _KT, default: _VT | _T = ...) -> _VT | _T: ...
    def popitem(self) -> tuple[_KT, _VT]: ...
    def setdefault(self, key: _KT, default: _VT | None = None) -> _VT: ...
    @classmethod
    def unique(cls, *a, **kw) -> Self: ...
    def update(self, dict_or_iterable, **kw) -> None: ...  # type: ignore[override]

class ManyToMany(dict[_KT, frozenset[_VT]]):
    data: dict[_KT, set[_VT]]
    inv: dict[_VT, set[_KT]]
    # def __contains__(self, key: _KT): ...
    def __delitem__(self, key: _KT) -> None: ...
    def __eq__(self, other): ...
    def __getitem__(self, key: _KT): ...
    def __init__(
        self, items: ManyToMany[_KT, _VT] | SupportsKeysAndGetItem[_KT, _VT] | tuple[_KT, _VT] | None = None
    ) -> None: ...
    def __iter__(self): ...
    def __len__(self): ...
    def __setitem__(self, key: _KT, vals: Iterable[_VT]) -> None: ...
    def add(self, key: _KT, val: _VT) -> None: ...
    def get(self, key: _KT, default: frozenset[_VT] = ...) -> frozenset[_VT]: ...  # type: ignore[override]
    def iteritems(self) -> Generator[tuple[_KT, _VT], None, None]: ...
    def keys(self): ...
    def remove(self, key: _KT, val: _VT) -> None: ...
    def replace(self, key: _KT, newkey: _KT) -> None: ...
    def update(self, iterable: ManyToMany[_KT, _VT] | SupportsKeysAndGetItem[_KT, _VT] | tuple[_KT, _VT]) -> None: ...  # type: ignore[override]

def subdict(d: dict[_KT, _VT], keep: Iterable[_KT] | None = None, drop: Iterable[_KT] | None = None) -> dict[_KT, _VT]: ...

class FrozenHashError(TypeError): ...  # undocumented

class FrozenDict(dict[_KT, _VT]):
    __slots__ = ("_hash",)
    def __copy__(self) -> Self: ...
    @classmethod
    def fromkeys(cls, keys: Iterable[_KT], value: _VT | None = None) -> Self: ...  # type: ignore[override]
    def updated(self, *a, **kw) -> Self: ...
    def __ior__(self, *a, **kw) -> NoReturn: ...  # type: ignore[misc] # noqa: Y034 # Signature conflicts with superclass
    def __setitem__(self, *a, **kw) -> NoReturn: ...
    def __delitem__(self, *a, **kw) -> NoReturn: ...
    def update(self, *a, **kw) -> NoReturn: ...
    def pop(self, *a, **kw) -> NoReturn: ...
    def popitem(self, *a, **kw) -> NoReturn: ...
    def setdefault(self, *a, **kw) -> NoReturn: ...
    def clear(self, *a, **kw) -> NoReturn: ...

__all__ = ["MultiDict", "OMD", "OrderedMultiDict", "OneToOne", "ManyToMany", "subdict", "FrozenDict"]



from _typeshed import Incomplete

from braintree.error_result import ErrorResult
from braintree.successful_result import SuccessfulResult

class OAuthGateway:
    gateway: Incomplete
    config: Incomplete
    def __init__(self, gateway) -> None: ...
    def create_token_from_code(self, params: dict[str, Incomplete]) -> SuccessfulResult | ErrorResult: ...
    def create_token_from_refresh_token(self, params: dict[str, Incomplete]) -> SuccessfulResult | ErrorResult: ...
    def revoke_access_token(self, access_token: str) -> type[SuccessfulResult] | ErrorResult: ...
    def connect_url(self, raw_params: dict[str, Incomplete]) -> str: ...



from collections.abc import Callable
from typing import Final
from typing_extensions import LiteralString, Unpack

from .watch import Watch

__version__: Final[LiteralString]

watch: Watch
unwatch: Final[Callable[[Unpack[tuple[object, ...]]], None]]



from _typeshed import SupportsRead
from pathlib import Path

from PIL import Image

from ..._types import Ink
from ..styledpil import StyledPilImage

class QRColorMask:
    back_color: Ink
    has_transparency: bool
    paint_color: Ink
    # image is not actually used by any of the initialize implementations in this project.
    def initialize(self, styledPilImage: StyledPilImage, image: Image.Image) -> None: ...
    def apply_mask(self, image: Image.Image, use_cache: bool = False) -> None: ...
    def get_fg_pixel(self, image: Image.Image, x: int, y: int) -> Ink: ...
    def get_bg_pixel(self, image: Image.Image, x: int, y: int) -> Ink: ...
    def interp_num(self, n1: int, n2: int, norm: float) -> int: ...
    def interp_color(self, col1: Ink, col2: Ink, norm: float) -> Ink: ...
    def extrap_num(self, n1: int, n2: int, interped_num: int) -> float | None: ...
    def extrap_color(self, col1: Ink, col2: Ink, interped_color: Ink) -> float | None: ...

class SolidFillColorMask(QRColorMask):
    front_color: Ink
    def __init__(self, back_color: Ink = (255, 255, 255), front_color: Ink = (0, 0, 0)) -> None: ...
    def apply_mask(self, image: Image.Image) -> None: ...  # type: ignore[override]

class RadialGradiantColorMask(QRColorMask):
    center_color: Ink
    edge_color: Ink
    def __init__(
        self, back_color: Ink = (255, 255, 255), center_color: Ink = (0, 0, 0), edge_color: Ink = (0, 0, 255)
    ) -> None: ...

class SquareGradiantColorMask(QRColorMask):
    center_color: Ink
    edge_color: Ink
    def __init__(
        self, back_color: Ink = (255, 255, 255), center_color: Ink = (0, 0, 0), edge_color: Ink = (0, 0, 255)
    ) -> None: ...

class HorizontalGradiantColorMask(QRColorMask):
    left_color: Ink
    right_color: Ink
    def __init__(
        self, back_color: Ink = (255, 255, 255), left_color: Ink = (0, 0, 0), right_color: Ink = (0, 0, 255)
    ) -> None: ...

class VerticalGradiantColorMask(QRColorMask):
    top_color: Ink
    bottom_color: Ink
    def __init__(
        self, back_color: Ink = (255, 255, 255), top_color: Ink = (0, 0, 0), bottom_color: Ink = (0, 0, 255)
    ) -> None: ...

class ImageColorMask(QRColorMask):
    color_img: Ink
    def __init__(
        self,
        back_color: Ink = (255, 255, 255),
        color_mask_path: str | bytes | Path | SupportsRead[bytes] | None = None,
        color_mask_image: Image.Image | None = None,
    ) -> None: ...
    paint_color: Ink



from types import FrameType
from typing import Any
from typing_extensions import TypeAlias

from gunicorn.workers.base_async import AsyncWorker

from .._types import _AddressType

GreenSocket: TypeAlias = Any  # eventlet GreenSocket class

EVENTLET_WSGI_LOCAL: Any  # eventlet local instance
EVENTLET_ALREADY_HANDLED: bool | None

def patch_sendfile() -> None: ...

class EventletWorker(AsyncWorker):
    def patch(self) -> None: ...
    def is_already_handled(self, respiter: object) -> bool: ...
    def init_process(self) -> None: ...
    def handle_quit(self, sig: int, frame: FrameType | None) -> None: ...
    def handle_usr1(self, sig: int, frame: FrameType | None) -> None: ...
    def timeout_ctx(self) -> None: ...
    def handle(self, listener: GreenSocket, client: GreenSocket, addr: _AddressType) -> None: ...
    def run(self) -> None: ...



from _typeshed import Incomplete
from typing import ClassVar, Final

from authlib.jose.rfc7517 import Key

POSSIBLE_UNSAFE_KEYS: Final[tuple[bytes, ...]]

class OctKey(Key):
    kty: str
    REQUIRED_JSON_FIELDS: ClassVar[list[str]]
    raw_key: Incomplete
    def __init__(self, raw_key=None, options=None) -> None: ...
    @property
    def public_only(self): ...
    def get_op_key(self, operation): ...
    def load_raw_key(self) -> None: ...
    def load_dict_key(self) -> None: ...
    def as_dict(self, is_private: bool = False, **params): ...
    @classmethod
    def validate_raw_key(cls, key): ...
    @classmethod
    def import_key(cls, raw, options=None): ...
    @classmethod
    def generate_key(cls, key_size: int = 256, options=None, is_private: bool = True): ...



from _typeshed import Incomplete
from collections.abc import Mapping
from typing import Final

__version__: Final[str]

class PDFError(Exception): ...

__InternalName__: Final[str]
__RefOnly__: Final[str]
__Comment__: Final[str]
BasicFonts: Final[str]
Pages: Final[str]
PDF_VERSION_DEFAULT: Final[tuple[int, int]]
PDF_SUPPORT_VERSION: Final[Mapping[str, tuple[int, int]]]

def pdfdocEnc(x): ...
def format(element, document, toplevel: int = 0): ...
def xObjectName(externalname): ...

formName = xObjectName

class NoEncryption:
    def encode(self, t): ...
    def prepare(self, document) -> None: ...
    def register(self, objnum, version) -> None: ...
    def info(self) -> None: ...

class PDFObject: ...

class DummyDoc(PDFObject):
    encrypt: Incomplete

class PDFDocument(PDFObject):
    defaultStreamFilters: Incomplete
    encrypt: Incomplete
    objectcounter: int
    shadingCounter: int
    inObject: Incomplete
    pageCounter: int
    invariant: Incomplete
    idToObjectNumberAndVersion: Incomplete
    idToObject: Incomplete
    idToOffset: Incomplete
    numberToId: Incomplete
    outline: Incomplete
    info: Incomplete
    fontMapping: Incomplete
    delayedFonts: Incomplete
    def __init__(
        self, dummyoutline: int = 0, compression=1, invariant=0, filename=None, pdfVersion=(1, 3), lang=None
    ) -> None: ...
    compression: Incomplete
    def setCompression(self, onoff) -> None: ...
    def ensureMinPdfVersion(self, *keys) -> None: ...
    def updateSignature(self, thing) -> None: ...
    def ID(self): ...
    def SaveToFile(self, filename, canvas) -> None: ...
    def GetPDFData(self, canvas): ...
    def inPage(self) -> None: ...
    def inForm(self) -> None: ...
    def getInternalFontName(self, psfontname): ...
    def thisPageName(self): ...
    def thisPageRef(self): ...
    def addPage(self, page) -> None: ...
    def addForm(self, name, form) -> None: ...
    def annotationName(self, externalname): ...
    def addAnnotation(self, name, annotation) -> None: ...
    def refAnnotation(self, name): ...
    def addShading(self, shading): ...
    def addColor(self, cmyk): ...
    def setTitle(self, title) -> None: ...
    def setAuthor(self, author) -> None: ...
    def setSubject(self, subject) -> None: ...
    def setCreator(self, creator) -> None: ...
    def setProducer(self, producer) -> None: ...
    def setKeywords(self, keywords) -> None: ...
    def setDateFormatter(self, dateFormatter) -> None: ...
    def getAvailableFonts(self): ...
    __accum__: Incomplete
    def format(self): ...
    def hasForm(self, name): ...
    def getFormBBox(self, name, boxType: str = "MediaBox"): ...
    def getXObjectName(self, name): ...
    def xobjDict(self, formnames): ...
    def Reference(self, obj, name=None): ...

PDFtrue: str
PDFfalse: str
PDFnull: str

class PDFText(PDFObject):
    t: Incomplete
    enc: Incomplete
    def __init__(self, t, enc: str = "utf-8") -> None: ...
    def format(self, document): ...

def PDFnumber(n): ...

class PDFString(PDFObject):
    unicodeEncValid: bool
    s: Incomplete
    escape: Incomplete
    enc: Incomplete
    def __init__(self, s, escape: int = 1, enc: str = "auto") -> None: ...
    def format(self, document): ...

def PDFName(data, lo="!", hi="~"): ...

class PDFDictionary(PDFObject):
    multiline: bool
    dict: Incomplete
    def __init__(self, dict=None) -> None: ...
    def __setitem__(self, name, value) -> None: ...
    def __getitem__(self, a): ...
    def __contains__(self, a) -> bool: ...
    def Reference(self, name, document) -> None: ...
    def format(self, document, IND: bytes = b"\n "): ...
    def copy(self): ...
    def normalize(self) -> None: ...

class checkPDFNames:
    names: Incomplete
    def __init__(self, *names) -> None: ...
    def __call__(self, value): ...

def checkPDFBoolean(value): ...

class CheckedPDFDictionary(PDFDictionary):
    validate: Incomplete
    def __init__(self, dict=None, validate=None) -> None: ...
    def __setitem__(self, name, value) -> None: ...

class ViewerPreferencesPDFDictionary(CheckedPDFDictionary):
    validate: Incomplete

class PDFStreamFilterZCompress:
    pdfname: str
    def encode(self, text): ...
    def decode(self, encoded): ...

PDFZCompress: Incomplete

class PDFStreamFilterBase85Encode:
    pdfname: str
    def encode(self, text): ...
    def decode(self, text): ...

PDFBase85Encode: Incomplete

class PDFStream(PDFObject):
    __RefOnly__: int
    dictionary: Incomplete
    content: Incomplete
    filters: Incomplete
    def __init__(self, dictionary=None, content=None, filters=None) -> None: ...
    def format(self, document): ...

def teststream(content=None): ...

teststreamcontent: str

class PDFArray(PDFObject):
    multiline: bool
    sequence: Incomplete
    def __init__(self, sequence) -> None: ...
    def References(self, document) -> None: ...
    def format(self, document, IND: bytes = b"\n "): ...

class PDFArrayCompact(PDFArray):
    multiline: bool

class PDFIndirectObject(PDFObject):
    __RefOnly__: int
    name: Incomplete
    content: Incomplete
    def __init__(self, name, content) -> None: ...
    def format(self, document): ...

class PDFObjectReference(PDFObject):
    name: Incomplete
    def __init__(self, name) -> None: ...
    def format(self, document): ...

class PDFFile(PDFObject):
    strings: Incomplete
    write: Incomplete
    offset: int
    def __init__(self, pdfVersion=(1, 3)) -> None: ...
    def closeOrReset(self) -> None: ...
    def add(self, s): ...
    def format(self, document): ...

class PDFCrossReferenceSubsection(PDFObject):
    firstentrynumber: Incomplete
    idsequence: Incomplete
    def __init__(self, firstentrynumber, idsequence) -> None: ...
    def format(self, document): ...

class PDFCrossReferenceTable(PDFObject):
    sections: Incomplete
    def __init__(self) -> None: ...
    def addsection(self, firstentry, ids) -> None: ...
    def format(self, document): ...

class PDFTrailer(PDFObject):
    startxref: Incomplete
    def __init__(self, startxref, Size=None, Prev=None, Root=None, Info=None, ID=None, Encrypt=None) -> None: ...
    def format(self, document): ...

class PDFCatalog(PDFObject):
    __Comment__: str
    __RefOnly__: int
    __Defaults__: Incomplete
    __NoDefault__: Incomplete
    __Refs__ = __NoDefault__
    def format(self, document): ...
    def showOutline(self) -> None: ...
    def showFullScreen(self) -> None: ...
    PageLayout: Incomplete
    def setPageLayout(self, layout) -> None: ...
    PageMode: Incomplete
    def setPageMode(self, mode) -> None: ...
    def check_format(self, document) -> None: ...

class PDFPages(PDFCatalog):
    __Comment__: str
    __RefOnly__: int
    __Defaults__: Incomplete
    __NoDefault__: Incomplete
    __Refs__: Incomplete
    pages: Incomplete
    def __init__(self) -> None: ...
    def __getitem__(self, item): ...
    def addPage(self, page) -> None: ...
    Kids: Incomplete
    Count: Incomplete
    def check_format(self, document) -> None: ...

class PDFPage(PDFCatalog):
    __Comment__: str
    Override_default_compilation: int
    __RefOnly__: int
    __Defaults__: Incomplete
    __NoDefault__: Incomplete
    __Refs__: Incomplete
    pagewidth: int
    pageheight: int
    stream: Incomplete
    hasImages: int
    compression: int
    XObjects: Incomplete
    Trans: Incomplete
    def __init__(self) -> None: ...
    def setCompression(self, onoff) -> None: ...
    def setStream(self, code) -> None: ...
    def setPageTransition(self, tranDict) -> None: ...
    MediaBox: Incomplete
    Annots: Incomplete
    Contents: Incomplete
    Resources: Incomplete
    Parent: Incomplete
    def check_format(self, document) -> None: ...

class DuplicatePageLabelPage(Exception): ...

class PDFPageLabels(PDFCatalog):
    __comment__: Incomplete
    __RefOnly__: int
    __Defaults__: Incomplete
    __NoDefault__: Incomplete
    __Refs__: Incomplete
    labels: Incomplete
    def __init__(self) -> None: ...
    def addPageLabel(self, page, label) -> None: ...
    Nums: Incomplete
    def format(self, document): ...

class PDFPageLabel(PDFCatalog):
    __Comment__: Incomplete
    __RefOnly__: int
    __Defaults__: Incomplete
    __NoDefault__: Incomplete
    __convertible__: str
    ARABIC: str
    ROMAN_UPPER: str
    ROMAN_LOWER: str
    LETTERS_UPPER: str
    LETTERS_LOWER: str
    S: Incomplete
    St: Incomplete
    P: Incomplete
    def __init__(self, style=None, start=None, prefix=None) -> None: ...
    def __lt__(self, oth): ...

def testpage(document) -> None: ...

DUMMYOUTLINE: str

class PDFOutlines0(PDFObject):
    __Comment__: str
    text: Incomplete
    __RefOnly__: int
    def format(self, document): ...

class OutlineEntryObject(PDFObject):
    Title: Incomplete
    Dest: Incomplete
    Parent: Incomplete
    Prev: Incomplete
    Next: Incomplete
    First: Incomplete
    Last: Incomplete
    Count: Incomplete
    def format(self, document): ...

class PDFOutlines(PDFObject):
    mydestinations: Incomplete
    ready: Incomplete
    counter: int
    currentlevel: int
    destinationnamestotitles: Incomplete
    destinationstotitles: Incomplete
    levelstack: Incomplete
    buildtree: Incomplete
    closedict: Incomplete
    def __init__(self) -> None: ...
    def addOutlineEntry(self, destinationname, level: int = 0, title=None, closed=None) -> None: ...
    def setDestinations(self, destinationtree) -> None: ...
    def format(self, document): ...
    def setNames(self, canvas, *nametree) -> None: ...
    def setNameList(self, canvas, nametree) -> None: ...
    def translateNames(self, canvas, object): ...
    first: Incomplete
    count: int
    def prepare(self, document, canvas) -> None: ...
    def maketree(self, document, destinationtree, Parent=None, toplevel: int = 0): ...

def count(tree, closedict=None): ...

class PDFInfo(PDFObject):
    producer: Incomplete
    creator: str
    title: str
    author: str
    subject: str
    keywords: str
    trapped: str
    def __init__(self) -> None: ...
    def digest(self, md5object) -> None: ...
    def format(self, document): ...
    def copy(self): ...

class Annotation(PDFObject):
    defaults: Incomplete
    required: Incomplete
    permitted: Incomplete
    def cvtdict(self, d, escape: int = 1): ...
    def AnnotationDict(self, **kw): ...
    def Dict(self) -> None: ...
    def format(self, document): ...

class FreeTextAnnotation(Annotation):
    permitted: Incomplete
    Rect: Incomplete
    Contents: Incomplete
    DA: Incomplete
    otherkw: Incomplete
    def __init__(self, Rect, Contents, DA, **kw) -> None: ...
    def Dict(self): ...

class LinkAnnotation(Annotation):
    permitted: Incomplete
    Border: Incomplete
    Rect: Incomplete
    Contents: Incomplete
    Destination: Incomplete
    otherkw: Incomplete
    def __init__(self, Rect, Contents, Destination, Border: str = "[0 0 1]", **kw) -> None: ...
    def dummyDictString(self): ...
    def Dict(self): ...

class HighlightAnnotation(Annotation):
    permitted: Incomplete
    Rect: Incomplete
    Contents: Incomplete
    otherkw: Incomplete
    QuadPoints: Incomplete
    Color: Incomplete
    def __init__(self, Rect, Contents, QuadPoints, Color=[0.83, 0.89, 0.95], **kw) -> None: ...
    def cvtdict(self, d, escape: int = 1): ...
    def Dict(self): ...

class TextAnnotation(HighlightAnnotation):
    permitted: Incomplete
    def __init__(self, Rect, Contents, **kw) -> None: ...
    def Dict(self): ...

def rect_to_quad(Rect): ...

class PDFRectangle(PDFObject):
    def __init__(self, llx, lly, urx, ury) -> None: ...
    def format(self, document): ...

class PDFDate(PDFObject):
    dateFormatter: Incomplete
    def __init__(self, ts=None, dateFormatter=None) -> None: ...
    def format(self, doc): ...

class Destination(PDFObject):
    representation: None
    page: None
    name: Incomplete
    fmt: Incomplete
    def __init__(self, name) -> None: ...
    def format(self, document): ...
    def xyz(self, left, top, zoom) -> None: ...
    def fit(self) -> None: ...
    def fitb(self) -> None: ...
    def fith(self, top) -> None: ...
    def fitv(self, left) -> None: ...
    def fitbh(self, top) -> None: ...
    def fitbv(self, left) -> None: ...
    def fitr(self, left, bottom, right, top) -> None: ...
    def setPage(self, page) -> None: ...

class PDFDestinationXYZ(PDFObject):
    typename: str
    page: Incomplete
    top: Incomplete
    zoom: Incomplete
    left: Incomplete
    def __init__(self, page, left, top, zoom) -> None: ...
    def format(self, document): ...

class PDFDestinationFit(PDFObject):
    typename: str
    page: Incomplete
    def __init__(self, page) -> None: ...
    def format(self, document): ...

class PDFDestinationFitB(PDFDestinationFit):
    typename: str

class PDFDestinationFitH(PDFObject):
    typename: str
    page: Incomplete
    top: Incomplete
    def __init__(self, page, top) -> None: ...
    def format(self, document): ...

class PDFDestinationFitBH(PDFDestinationFitH):
    typename: str

class PDFDestinationFitV(PDFObject):
    typename: str
    page: Incomplete
    left: Incomplete
    def __init__(self, page, left) -> None: ...
    def format(self, document): ...

class PDFDestinationFitBV(PDFDestinationFitV):
    typename: str

class PDFDestinationFitR(PDFObject):
    typename: str
    page: Incomplete
    left: Incomplete
    bottom: Incomplete
    right: Incomplete
    top: Incomplete
    def __init__(self, page, left, bottom, right, top) -> None: ...
    def format(self, document): ...

class PDFResourceDictionary(PDFObject):
    ProcSet: Incomplete
    def __init__(self, **kwds) -> None: ...
    stdprocs: Incomplete
    dict_attributes: Incomplete
    def allProcs(self) -> None: ...
    def basicProcs(self) -> None: ...
    Font: Incomplete
    def basicFonts(self) -> None: ...
    def setColorSpace(self, colorsUsed) -> None: ...
    def setShading(self, shadingUsed) -> None: ...
    def format(self, document): ...

class PDFType1Font(PDFObject):
    __RefOnly__: int
    name_attributes: Incomplete
    Type: str
    Subtype: str
    local_attributes: Incomplete
    def format(self, document): ...

class PDFTrueTypeFont(PDFType1Font):
    Subtype: str

class PDFFormXObject(PDFObject):
    XObjects: Incomplete
    Annots: Incomplete
    BBox: Incomplete
    Matrix: Incomplete
    Contents: Incomplete
    stream: Incomplete
    Resources: Incomplete
    hasImages: int
    compression: int
    lowerx: Incomplete
    lowery: Incomplete
    upperx: Incomplete
    uppery: Incomplete
    def __init__(self, lowerx, lowery, upperx, uppery) -> None: ...
    def setStreamList(self, data) -> None: ...
    def BBoxList(self): ...
    def format(self, document): ...

class PDFPostScriptXObject(PDFObject):
    content: Incomplete
    def __init__(self, content=None) -> None: ...
    def format(self, document): ...

class PDFImageXObject(PDFObject):
    name: Incomplete
    width: int
    height: int
    bitsPerComponent: int
    colorSpace: str
    streamContent: str
    mask: Incomplete
    def __init__(self, name, source=None, mask=None) -> None: ...
    def loadImageFromA85(self, source): ...
    def loadImageFromJPEG(self, imageFile): ...
    def loadImageFromRaw(self, source): ...
    def loadImageFromSRC(self, im) -> None: ...
    def format(self, document): ...

class PDFSeparationCMYKColor:
    cmyk: Incomplete
    def __init__(self, cmyk) -> None: ...
    def value(self): ...

class PDFFunction(PDFObject):
    defaults: Incomplete
    required: Incomplete
    permitted: Incomplete
    def FunctionDict(self, **kw): ...
    def Dict(self, document) -> None: ...
    def format(self, document): ...

class PDFExponentialFunction(PDFFunction):
    defaults: Incomplete
    required: Incomplete
    permitted: Incomplete
    C0: Incomplete
    C1: Incomplete
    N: Incomplete
    otherkw: Incomplete
    def __init__(self, C0, C1, N, **kw) -> None: ...
    def Dict(self, document): ...

class PDFStitchingFunction(PDFFunction):
    required: Incomplete
    permitted: Incomplete
    Functions: Incomplete
    Bounds: Incomplete
    Encode: Incomplete
    otherkw: Incomplete
    def __init__(self, Functions, Bounds, Encode, **kw) -> None: ...
    def Dict(self, document): ...

class PDFShading(PDFObject):
    required: Incomplete
    permitted: Incomplete
    def ShadingDict(self, **kw): ...
    def Dict(self, document) -> None: ...
    def format(self, document): ...

class PDFFunctionShading(PDFShading):
    required: Incomplete
    permitted: Incomplete
    Function: Incomplete
    ColorSpace: Incomplete
    otherkw: Incomplete
    def __init__(self, Function, ColorSpace, **kw) -> None: ...
    def Dict(self, document): ...

class PDFAxialShading(PDFShading):
    required: Incomplete
    permitted: Incomplete
    Coords: Incomplete
    Function: Incomplete
    ColorSpace: Incomplete
    otherkw: Incomplete
    def __init__(self, x0, y0, x1, y1, Function, ColorSpace, **kw) -> None: ...
    def Dict(self, document): ...

class PDFRadialShading(PDFShading):
    required: Incomplete
    permitted: Incomplete
    Coords: Incomplete
    Function: Incomplete
    ColorSpace: Incomplete
    otherkw: Incomplete
    def __init__(self, x0, y0, r0, x1, y1, r1, Function, ColorSpace, **kw) -> None: ...
    def Dict(self, document): ...

class XMP(PDFStream):
    def __init__(self, path=None, creator=None) -> None: ...
    def makeContent(self, doc): ...
    # Param name is changed from the base class:
    def format(self, doc): ...



from collections.abc import Callable, Iterable
from datetime import date, datetime, timedelta
from time import _TimeTuple, struct_time
from typing import Any, NamedTuple, TypeVar, overload
from typing_extensions import TypeAlias

from webob._types import AsymmetricProperty, AsymmetricPropertyWithDelete, SymmetricProperty, SymmetricPropertyWithDelete
from webob.byterange import ContentRange, Range
from webob.etag import IfRange, IfRangeDate

_DefaultT = TypeVar("_DefaultT")
_GetterReturnType = TypeVar("_GetterReturnType")
_SetterValueType = TypeVar("_SetterValueType")
_ConvertedGetterReturnType = TypeVar("_ConvertedGetterReturnType")
_ConvertedSetterValueType = TypeVar("_ConvertedSetterValueType")
_DescriptorT = TypeVar("_DescriptorT", bound=AsymmetricPropertyWithDelete[Any, Any])

_StringProperty: TypeAlias = SymmetricPropertyWithDelete[str | None]
_ListProperty: TypeAlias = AsymmetricPropertyWithDelete[tuple[str, ...] | None, Iterable[str] | str | None]
_DateProperty: TypeAlias = AsymmetricPropertyWithDelete[
    datetime | None, date | datetime | timedelta | _TimeTuple | struct_time | float | str | None
]
_ContentRangeParams: TypeAlias = (
    ContentRange
    | list[int]
    | list[None]
    | list[int | None]
    | tuple[int, int]
    | tuple[None, None]
    | tuple[int, int, int | None]
    | tuple[None, None, int | None]
    | str
    | None
)

@overload
def environ_getter(key: str, *, rfc_section: str | None = None) -> SymmetricProperty[Any]: ...
@overload
def environ_getter(key: str, default: None, rfc_section: str | None = None) -> SymmetricPropertyWithDelete[Any | None]: ...
@overload
def environ_getter(
    key: str, default: _DefaultT, rfc_section: str | None = None
) -> AsymmetricPropertyWithDelete[Any | _DefaultT, Any | _DefaultT | None]: ...
@overload
def environ_decoder(key: str, *, rfc_section: str | None = None, encattr: str | None = None) -> SymmetricProperty[str]: ...
@overload
def environ_decoder(
    key: str, default: str, rfc_section: str | None = None, encattr: str | None = None
) -> AsymmetricPropertyWithDelete[str, str | None]: ...
@overload
def environ_decoder(
    key: str, default: None, rfc_section: str | None = None, encattr: str | None = None
) -> SymmetricPropertyWithDelete[str | None]: ...
def upath_property(key: str) -> SymmetricProperty[str]: ...
def deprecated_property(attr: _DescriptorT, name: str, text: str, version: str) -> _DescriptorT: ...
def header_getter(header: str, rfc_section: str) -> _StringProperty: ...
@overload
def converter(
    prop: AsymmetricPropertyWithDelete[_GetterReturnType, _SetterValueType],
    parse: Callable[[_GetterReturnType], _ConvertedGetterReturnType],
    serialize: Callable[[_ConvertedSetterValueType], _SetterValueType],
    convert_name: str | None = None,
) -> AsymmetricPropertyWithDelete[_ConvertedGetterReturnType, _ConvertedSetterValueType | None]: ...
@overload
def converter(
    prop: AsymmetricProperty[_GetterReturnType, _SetterValueType],
    parse: Callable[[_GetterReturnType], _ConvertedGetterReturnType],
    serialize: Callable[[_ConvertedSetterValueType], _SetterValueType],
    convert_name: str | None = None,
) -> AsymmetricProperty[_ConvertedGetterReturnType, _ConvertedSetterValueType | None]: ...
def list_header(header: str, rfc_section: str) -> _ListProperty: ...
def parse_list(value: str | None) -> tuple[str, ...] | None: ...
def serialize_list(value: Iterable[str] | str) -> str: ...
def converter_date(prop: _StringProperty) -> _DateProperty: ...
def date_header(header: str, rfc_section: str) -> _DateProperty: ...
def parse_etag_response(value: str | None, strong: bool = False) -> str | None: ...
def serialize_etag_response(value: tuple[str, bool] | str) -> str: ...
def serialize_if_range(value: IfRange | IfRangeDate | datetime | date | str) -> str | None: ...
def parse_range(value: str | None) -> Range | None: ...
def serialize_range(value: tuple[int, int | None] | list[int | None] | list[int] | str | None) -> str | None: ...
def parse_int(value: str | None) -> int | None: ...
def parse_int_safe(value: str | None) -> int | None: ...

serialize_int: Callable[[int], str]

def parse_content_range(value: str | None) -> ContentRange | None: ...
def serialize_content_range(value: _ContentRangeParams) -> str | None: ...
def parse_auth_params(params: str) -> dict[str, str]: ...

known_auth_schemes: dict[str, None]

class _authorization(NamedTuple):
    authtype: str
    params: dict[str, str] | str

def parse_auth(val: str | None) -> _authorization | None: ...
def serialize_auth(val: tuple[str, dict[str, str] | str] | list[Any] | str | None) -> str | None: ...



"""Preconfigured converters for orjson."""

from base64 import b85decode, b85encode
from datetime import date, datetime
from enum import Enum
from functools import partial
from typing import Any, Type, TypeVar, Union

from orjson import dumps, loads

from .._compat import AbstractSet, is_mapping
from ..cols import is_namedtuple, namedtuple_unstructure_factory
from ..converters import BaseConverter, Converter
from ..fns import identity
from ..strategies import configure_union_passthrough
from . import wrap

T = TypeVar("T")


class OrjsonConverter(Converter):
    def dumps(self, obj: Any, unstructure_as: Any = None, **kwargs: Any) -> bytes:
        return dumps(self.unstructure(obj, unstructure_as=unstructure_as), **kwargs)

    def loads(self, data: Union[bytes, bytearray, memoryview, str], cl: Type[T]) -> T:
        return self.structure(loads(data), cl)


def configure_converter(converter: BaseConverter):
    """
    Configure the converter for use with the orjson library.

    * bytes are serialized as base85 strings
    * datetimes and dates are passed through to be serialized as RFC 3339 by orjson
    * typed namedtuples are serialized as lists
    * sets are serialized as lists
    * string enum mapping keys have special handling
    * mapping keys are coerced into strings when unstructuring

    .. versionchanged: 24.1.0
        Add support for typed namedtuples.
    """
    converter.register_unstructure_hook(
        bytes, lambda v: (b85encode(v) if v else b"").decode("utf8")
    )
    converter.register_structure_hook(bytes, lambda v, _: b85decode(v))

    converter.register_structure_hook(datetime, lambda v, _: datetime.fromisoformat(v))
    converter.register_structure_hook(date, lambda v, _: date.fromisoformat(v))

    def gen_unstructure_mapping(cl: Any, unstructure_to=None):
        key_handler = str
        args = getattr(cl, "__args__", None)
        if args:
            if issubclass(args[0], str) and issubclass(args[0], Enum):

                def key_handler(v):
                    return v.value

            else:
                # It's possible the handler for the key type has been overridden.
                # (For example base85 encoding for bytes.)
                # In that case, we want to use the override.

                kh = converter.get_unstructure_hook(args[0])
                if kh != identity:
                    key_handler = kh

        return converter.gen_unstructure_mapping(
            cl, unstructure_to=unstructure_to, key_handler=key_handler
        )

    converter._unstructure_func.register_func_list(
        [
            (is_mapping, gen_unstructure_mapping, True),
            (
                is_namedtuple,
                partial(namedtuple_unstructure_factory, unstructure_to=tuple),
                "extended",
            ),
        ]
    )
    configure_union_passthrough(Union[str, bool, int, float, None], converter)


@wrap(OrjsonConverter)
def make_converter(*args: Any, **kwargs: Any) -> OrjsonConverter:
    kwargs["unstruct_collection_overrides"] = {
        AbstractSet: list,
        **kwargs.get("unstruct_collection_overrides", {}),
    }
    res = OrjsonConverter(*args, **kwargs)
    configure_converter(res)

    return res



import unittest
from _typeshed import Incomplete
from configparser import ConfigParser
from typing import Final, Literal

__version__: Final[str]

def haveRenderPM(): ...
def isWritable(D): ...

RL_HOME: Incomplete
testsFolder: Incomplete

DEJAVUSANS: tuple[
    Literal["DejaVuSans"], Literal["DejaVuSans-Bold"], Literal["DejaVuSans-Oblique"], Literal["DejaVuSans-BoldOblique"]
] = ...

def haveDejaVu() -> bool: ...
def setOutDir(name): ...
def mockUrlRead(name): ...
def outputfile(fn): ...
def printLocation(depth: int = 1) -> None: ...
def makeSuiteForClasses(*classes, testMethodPrefix=None): ...
def getCVSEntries(folder, files: int = 1, folders: int = 0): ...

class ExtConfigParser(ConfigParser):
    pat: Incomplete
    def getstringlist(self, section, option): ...

class GlobDirectoryWalker:
    index: int
    pattern: Incomplete
    stack: Incomplete
    files: Incomplete
    directory: Incomplete
    def __init__(self, directory, pattern: str = "*") -> None: ...
    def __getitem__(self, index): ...
    def filterFiles(self, folder, files): ...

class RestrictedGlobDirectoryWalker(GlobDirectoryWalker):
    ignorePatterns: Incomplete
    def __init__(self, directory, pattern: str = "*", ignore=None) -> None: ...
    def filterFiles(self, folder, files): ...

class CVSGlobDirectoryWalker(GlobDirectoryWalker):
    def filterFiles(self, folder, files): ...

class SecureTestCase(unittest.TestCase):
    def setUp(self) -> None: ...
    def tearDown(self) -> None: ...

class NearTestCase(unittest.TestCase):
    @staticmethod
    def assertNear(a, b, accuracy: float = 1e-05) -> None: ...

class ScriptThatMakesFileTest(unittest.TestCase):
    scriptDir: Incomplete
    scriptName: Incomplete
    outFileName: Incomplete
    verbose: Incomplete
    def __init__(self, scriptDir, scriptName, outFileName, verbose: int = 0) -> None: ...
    cwd: Incomplete
    def setUp(self) -> None: ...
    def tearDown(self) -> None: ...
    def runTest(self) -> None: ...

def equalStrings(a, b, enc: str = "utf8"): ...
def eqCheck(r, x) -> None: ...
def rlextraNeeded(): ...
def rlSkipIf(cond, reason, __module__=None): ...
def rlSkipUnless(cond, reason, __module__=None): ...
def rlSkip(reason, __module__=None): ...



from _typeshed import Incomplete

from authlib.oauth2.rfc7662 import IntrospectionEndpoint

class JWTIntrospectionEndpoint(IntrospectionEndpoint):
    ENDPOINT_NAME: str
    issuer: Incomplete
    def __init__(self, issuer, server=None, *args, **kwargs) -> None: ...
    def create_endpoint_response(self, request): ...
    def authenticate_token(self, request, client): ...
    def create_introspection_payload(self, token): ...
    def get_jwks(self) -> None: ...
    def get_username(self, user_id: str) -> str: ...



import io
from collections.abc import Callable, Generator, Iterator
from typing_extensions import TypeAlias

from gunicorn.http.message import Request
from gunicorn.http.unreader import Unreader

class ChunkedReader:
    req: Request
    parser: Generator[bytes, None, None] | None
    buf: io.BytesIO

    def __init__(self, req: Request, unreader: Unreader) -> None: ...
    def read(self, size: int) -> bytes: ...
    def parse_trailers(self, unreader: Unreader, data: bytes) -> None: ...
    def parse_chunked(self, unreader: Unreader) -> Generator[bytes]: ...
    def parse_chunk_size(self, unreader: Unreader, data: bytes | None = None) -> tuple[int, bytes | None]: ...
    def get_data(self, unreader: Unreader, buf: io.BytesIO) -> None: ...

class LengthReader:
    unreader: Unreader
    length: int

    def __init__(self, unreader: Unreader, length: int) -> None: ...
    def read(self, size: int) -> bytes: ...

class EOFReader:
    unreader: Unreader
    buf: io.BytesIO
    finished: bool

    def __init__(self, unreader: Unreader) -> None: ...
    def read(self, size: int) -> bytes: ...

_ReaderType: TypeAlias = ChunkedReader | LengthReader | EOFReader

class Body:
    reader: _ReaderType
    buf: io.BytesIO

    def __init__(self, reader: _ReaderType) -> None: ...
    def __iter__(self) -> Iterator[bytes]: ...
    def __next__(self) -> bytes: ...
    next: Callable[[Body], bytes]
    def getsize(self, size: int | None) -> int: ...
    def read(self, size: int | None = None) -> bytes: ...
    def readline(self, size: int | None = None) -> bytes: ...
    def readlines(self, size: int | None = None) -> list[bytes]: ...



from _typeshed import Incomplete
from collections import defaultdict
from collections.abc import Collection, Generator
from typing import Final

from networkx.classes.digraph import DiGraph
from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["triadic_census", "is_triad", "all_triads", "triads_by_type", "triad_type"]

TRICODES: Final[tuple[int, ...]]
TRIAD_NAMES: Final[tuple[str, ...]]
TRICODE_TO_NAME: Final[dict[int, str]]

@_dispatchable
def triadic_census(G: DiGraph[_Node], nodelist: Collection[_Node] | None = None) -> dict[str, int]: ...
@_dispatchable
def is_triad(G: Graph[_Node]) -> bool: ...
@_dispatchable
def all_triads(G: DiGraph[_Node]) -> Generator[Incomplete, None, None]: ...
@_dispatchable
def triads_by_type(G: DiGraph[_Node]) -> defaultdict[Incomplete, list[Incomplete]]: ...
@_dispatchable
def triad_type(G: DiGraph[_Node]) -> str | None: ...



from __future__ import annotations

import os
from typing import Any, Dict, Generic, Iterable, Mapping, TypeVar, Union
from typing_extensions import Self, assert_type

###################################################################
# Note: tests for `dict.update()` are in `check_MutableMapping.py`.
###################################################################

# These do follow `__init__` overloads order:
# mypy and pyright have different opinions about this one:
# mypy raises: 'Need type annotation for "bad"'
# pyright is fine with it.
# https://github.com/python/mypy/issues/12358
# bad = dict()
good: dict[str, str] = dict()
assert_type(good, Dict[str, str])

assert_type(dict(arg=1), Dict[str, int])

_KT = TypeVar("_KT")
_VT = TypeVar("_VT")


class KeysAndGetItem(Generic[_KT, _VT]):
    data: dict[_KT, _VT]

    def __init__(self, data: dict[_KT, _VT]) -> None:
        self.data = data

    def keys(self) -> Iterable[_KT]:
        return self.data.keys()

    def __getitem__(self, __k: _KT) -> _VT:
        return self.data[__k]


kt1: KeysAndGetItem[int, str] = KeysAndGetItem({0: ""})
assert_type(dict(kt1), Dict[int, str])
dict(kt1, arg="a")  # type: ignore

kt2: KeysAndGetItem[str, int] = KeysAndGetItem({"": 0})
assert_type(dict(kt2, arg=1), Dict[str, int])


def test_iterable_tuple_overload(x: Iterable[tuple[int, str]]) -> dict[int, str]:
    return dict(x)


i1: Iterable[tuple[int, str]] = [(1, "a"), (2, "b")]
test_iterable_tuple_overload(i1)
dict(i1, arg="a")  # type: ignore

i2: Iterable[tuple[str, int]] = [("a", 1), ("b", 2)]
assert_type(dict(i2, arg=1), Dict[str, int])

i3: Iterable[str] = ["a.b"]
i4: Iterable[bytes] = [b"a.b"]
assert_type(dict(string.split(".") for string in i3), Dict[str, str])
assert_type(dict(string.split(b".") for string in i4), Dict[bytes, bytes])

dict(["foo", "bar", "baz"])  # type: ignore
dict([b"foo", b"bar", b"baz"])  # type: ignore

# Exploring corner cases of dict.get()
d_any: dict[str, Any] = {}
d_str: dict[str, str] = {}
any_value: Any = None
str_value = "value"
int_value = 1

assert_type(d_any["key"], Any)
assert_type(d_any.get("key"), Union[Any, None])
assert_type(d_any.get("key", None), Union[Any, None])
assert_type(d_any.get("key", any_value), Any)
assert_type(d_any.get("key", str_value), Any)
assert_type(d_any.get("key", int_value), Any)

assert_type(d_str["key"], str)
assert_type(d_str.get("key"), Union[str, None])
assert_type(d_str.get("key", None), Union[str, None])
# Pyright has str instead of Any here
assert_type(d_str.get("key", any_value), Any)  # pyright: ignore[reportAssertTypeFailure]
assert_type(d_str.get("key", str_value), str)
assert_type(d_str.get("key", int_value), Union[str, int])

# Now with context!
result: str
result = d_any["key"]
result = d_any.get("key")  # type: ignore[assignment]
result = d_any.get("key", None)  # type: ignore[assignment]
result = d_any.get("key", any_value)
result = d_any.get("key", str_value)
result = d_any.get("key", int_value)

result = d_str["key"]
result = d_str.get("key")  # type: ignore[assignment]
result = d_str.get("key", None)  # type: ignore[assignment]
# Pyright has str | None here, see https://github.com/microsoft/pyright/discussions/9570
result = d_str.get("key", any_value)  # pyright: ignore[reportAssignmentType]
result = d_str.get("key", str_value)
result = d_str.get("key", int_value)  # type: ignore[arg-type]


# Return values also make things weird

# Pyright doesn't have a version of no-any-return,
# and mypy doesn't have a type: ignore that pyright will ignore.
# def test1() -> str:
#     return d_any["key"]  # mypy: ignore[no-any-return]


def test2() -> str:
    return d_any.get("key")  # type: ignore[return-value]


# def test3() -> str:
#     return d_any.get("key", None)  # mypy: ignore[no-any-return]
#
#
# def test4() -> str:
#     return d_any.get("key", any_value)  # mypy: ignore[no-any-return]
#
#
# def test5() -> str:
#     return d_any.get("key", str_value)  # mypy: ignore[no-any-return]
#
#
# def test6() -> str:
#     return d_any.get("key", int_value)  # mypy: ignore[no-any-return]


def test7() -> str:
    return d_str["key"]


def test8() -> str:
    return d_str.get("key")  # type: ignore[return-value]


def test9() -> str:
    return d_str.get("key", None)  # type: ignore[return-value]


def test10() -> str:
    return d_str.get("key", any_value)  # type: ignore[no-any-return]


def test11() -> str:
    return d_str.get("key", str_value)


def test12() -> str:
    return d_str.get("key", int_value)  # type: ignore[arg-type]


# Tests for `dict.__(r)or__`.


class CustomDictSubclass(dict[_KT, _VT]):
    pass


class CustomMappingWithDunderOr(Mapping[_KT, _VT]):
    def __or__(self, other: Mapping[_KT, _VT]) -> dict[_KT, _VT]:
        return {}

    def __ror__(self, other: Mapping[_KT, _VT]) -> dict[_KT, _VT]:
        return {}

    def __ior__(self, other: Mapping[_KT, _VT]) -> Self:
        return self


def test_dict_dot_or(
    a: dict[int, int],
    b: CustomDictSubclass[int, int],
    c: dict[str, str],
    d: Mapping[int, int],
    e: CustomMappingWithDunderOr[str, str],
) -> None:
    # dict.__(r)or__ always returns a dict, even if called on a subclass of dict:
    assert_type(a | b, dict[int, int])
    assert_type(b | a, dict[int, int])

    assert_type(a | c, dict[Union[int, str], Union[int, str]])

    # arbitrary mappings are not accepted by `dict.__or__`;
    # it has to be a subclass of `dict`
    a | d  # type: ignore

    # but Mappings such as `os._Environ` or `CustomMappingWithDunderOr`,
    # which define `__ror__` methods that accept `dict`, are fine:
    assert_type(a | os.environ, dict[Union[str, int], Union[str, int]])
    assert_type(os.environ | a, dict[Union[str, int], Union[str, int]])

    assert_type(c | os.environ, dict[str, str])
    assert_type(c | e, dict[str, str])

    assert_type(os.environ | c, dict[str, str])
    assert_type(e | c, dict[str, str])

    # store "untainted" `CustomMappingWithDunderOr[str, str]` to test `__ior__` against ` dict[str, str]` later
    # Invalid `e |= a` causes pyright to join `Unknown` to `e`'s type
    f = e

    e |= c
    e |= a  # type: ignore

    c |= f

    c |= a  # type: ignore



from collections.abc import Callable, Generator
from typing import Any

def parse_iter(
    parsed: Any, /, *, revivers: dict[str, Callable[[list[Any]], Any]] | None = None  # parsed: Any from the signature.
) -> Generator[TypeError | IndexError | ValueError, Any, float | None]: ...
def parse(
    parsed: Any, /, *, revivers: dict[str, Callable[[Any], Any]] | None = None  # parsed: Any from the signature.
) -> Any: ...  # returns StopIteration.value if it is raised.



import logging

from hvac.api.vault_api_base import VaultApiBase

DEFAULT_MOUNT_POINT: str

logger: logging.Logger

class Gcp(VaultApiBase):
    def configure(
        self, credentials=None, google_certs_endpoint="https://www.googleapis.com/oauth2/v3/certs", mount_point="gcp"
    ): ...
    def read_config(self, mount_point="gcp"): ...
    def delete_config(self, mount_point="gcp"): ...
    def create_role(
        self,
        name,
        role_type,
        project_id,
        ttl=None,
        max_ttl=None,
        period=None,
        policies=None,
        bound_service_accounts=None,
        max_jwt_exp=None,
        allow_gce_inference=None,
        bound_zones=None,
        bound_regions=None,
        bound_instance_groups=None,
        bound_labels=None,
        mount_point="gcp",
    ): ...
    def edit_service_accounts_on_iam_role(self, name, add=None, remove=None, mount_point="gcp"): ...
    def edit_labels_on_gce_role(self, name, add=None, remove=None, mount_point="gcp"): ...
    def read_role(self, name, mount_point="gcp"): ...
    def list_roles(self, mount_point="gcp"): ...
    def delete_role(self, role, mount_point="gcp"): ...
    def login(self, role, jwt, use_token: bool = True, mount_point="gcp"): ...



from _typeshed import Incomplete

class JsonWebEncryption:
    REGISTERED_HEADER_PARAMETER_NAMES: Incomplete
    ALG_REGISTRY: Incomplete
    ENC_REGISTRY: Incomplete
    ZIP_REGISTRY: Incomplete
    def __init__(self, algorithms=None, private_headers=None) -> None: ...
    @classmethod
    def register_algorithm(cls, algorithm) -> None: ...
    def serialize_compact(self, protected, payload, key, sender_key=None): ...
    def serialize_json(self, header_obj, payload, keys, sender_key=None): ...
    def serialize(self, header, payload, key, sender_key=None): ...
    def deserialize_compact(self, s, key, decode=None, sender_key=None): ...
    def deserialize_json(self, obj, key, decode=None, sender_key=None): ...
    def deserialize(self, obj, key, decode=None, sender_key=None): ...
    @staticmethod
    def parse_json(obj): ...
    def get_header_alg(self, header): ...
    def get_header_enc(self, header): ...
    def get_header_zip(self, header): ...

def prepare_key(alg, header, key): ...



import threading
from _typeshed import Incomplete, SupportsReadline, Unused
from collections.abc import Callable
from contextlib import AbstractContextManager
from typing import Any, Generic, TypeVar, type_check_only
from typing_extensions import TypeAlias

from .io import _SupportsReadAndReadlineBytes, _SupportsWriteAndFlushBytes
from .startup import DAPQueue

_T = TypeVar("_T")
_F = TypeVar("_F", bound=Callable[..., Any])

_RequestID: TypeAlias = int
_EventBody: TypeAlias = Any  # arbitrary object, implicitly constrained by the event being sent
_JSONValue: TypeAlias = Any  # any object that can be handled by json.dumps/json.loads

class NotStoppedException(Exception): ...

class CancellationHandler:
    lock: threading.Lock
    reqs: list[_RequestID]
    in_flight_dap_thread: _RequestID | None
    in_flight_gdb_thread: _RequestID | None
    def starting(self, req: _RequestID) -> None: ...
    def done(self, req: _RequestID) -> None: ...  # req argument is not used
    def cancel(self, req: _RequestID) -> None: ...
    def interruptable_region(self, req: _RequestID | None) -> AbstractContextManager[None]: ...

class Server:
    in_stream: _SupportsReadAndReadlineBytes
    out_stream: _SupportsWriteAndFlushBytes
    child_stream: SupportsReadline[str]
    delay_events: list[tuple[str, _EventBody]]
    write_queue: DAPQueue[_JSONValue | None]
    read_queue: DAPQueue[_JSONValue | None]
    done: bool
    canceller: CancellationHandler
    config: dict[str, Incomplete]
    def __init__(
        self,
        in_stream: _SupportsReadAndReadlineBytes,
        out_stream: _SupportsWriteAndFlushBytes,
        child_stream: SupportsReadline[str],
    ) -> None: ...
    def main_loop(self) -> None: ...
    def send_event(self, event: str, body: _EventBody | None = None) -> None: ...
    def send_event_later(self, event: str, body: _EventBody | None = None) -> None: ...
    def shutdown(self) -> None: ...

def send_event(event: str, body: _EventBody | None = None) -> None: ...
@type_check_only
class _Wrapper:
    def __call__(self, func: _F) -> _F: ...

def request(name: str, *, response: bool = True, on_dap_thread: bool = False, expect_stopped: bool = True) -> _Wrapper: ...
def capability(name: str, value: bool = True) -> _Wrapper: ...
def client_bool_capability(name: str) -> bool: ...
def initialize(**args) -> dict[str, bool]: ...  # args is arbitrary values for Server.config
def terminate(**args: Unused) -> None: ...
def disconnect(*, terminateDebuggee: bool = False, **args: Unused): ...
def cancel(**args: Unused) -> None: ...

class Invoker:
    cmd: str
    def __init__(self, cmd: str) -> None: ...
    def __call__(self) -> None: ...

class Cancellable(Generic[_T]):
    fn: Callable[[], _T]
    result_q: DAPQueue[_T] | None
    req: _RequestID
    def __init__(self, fn: Callable[[], _T], result_q: DAPQueue[_T] | None = None) -> None: ...
    def __call__(self) -> None: ...

def send_gdb(cmd: str | Callable[[], object]) -> None: ...
def send_gdb_with_response(fn: str | Callable[[], _T]) -> _T: ...



from collections.abc import Iterator
from typing import Final

from ..lexer import Lexer, RegexLexer
from ..token import _TokenType

__all__ = [
    "JavascriptLexer",
    "KalLexer",
    "LiveScriptLexer",
    "DartLexer",
    "TypeScriptLexer",
    "LassoLexer",
    "ObjectiveJLexer",
    "CoffeeScriptLexer",
    "MaskLexer",
    "EarlGreyLexer",
    "JuttleLexer",
    "NodeConsoleLexer",
]

JS_IDENT_START: Final[str]
JS_IDENT_PART: Final[str]
JS_IDENT: Final[str]

class JavascriptLexer(RegexLexer): ...
class TypeScriptLexer(JavascriptLexer): ...
class KalLexer(RegexLexer): ...
class LiveScriptLexer(RegexLexer): ...
class DartLexer(RegexLexer): ...

class LassoLexer(RegexLexer):
    def get_tokens_unprocessed(self, text: str) -> Iterator[tuple[int, _TokenType, str]]: ...  # type: ignore[override]

class ObjectiveJLexer(RegexLexer): ...
class CoffeeScriptLexer(RegexLexer): ...
class MaskLexer(RegexLexer): ...
class EarlGreyLexer(RegexLexer): ...
class JuttleLexer(RegexLexer): ...
class NodeConsoleLexer(Lexer): ...



from _typeshed import SupportsItems, Unused
from collections.abc import Callable
from typing import Any, NamedTuple

class Option(NamedTuple):
    key: str
    default_value: Any  # Can be "any" type
    doc: str
    validator: Callable[[object], Unused]
    callback: Callable[[str, object], Unused] | None

class Options:
    def __init__(self, options: SupportsItems[str, Option]) -> None: ...
    # Accept and return arbitrary values
    def __setattr__(self, key: str, value: Any) -> None: ...
    def __getattr__(self, key: str) -> Any: ...

display_precision: Option
use_pygeos: Option
io_engine: Option
options: Options



import ctypes
import sys
from _typeshed import Incomplete
from collections.abc import Generator
from ctypes.wintypes import DWORD

from serial.tools.list_ports_common import ListPortInfo

if sys.platform == "win32":

    def ValidHandle(
        value: type[ctypes._CData] | None, func: ctypes._FuncPointer, arguments: tuple[ctypes._CData, ...]
    ) -> ctypes._CData: ...

    NULL: int
    HDEVINFO = ctypes.c_void_p
    LPCTSTR = ctypes.c_wchar_p
    PCTSTR = ctypes.c_wchar_p
    PTSTR = ctypes.c_wchar_p
    LPDWORD: ctypes._Pointer[DWORD]
    PDWORD: ctypes._Pointer[DWORD]
    LPBYTE = ctypes.c_void_p
    PBYTE = ctypes.c_void_p
    ACCESS_MASK = DWORD
    REGSAM = ACCESS_MASK

    class GUID(ctypes.Structure):
        Data1: ctypes._CField[Incomplete, Incomplete, Incomplete]
        Data2: ctypes._CField[Incomplete, Incomplete, Incomplete]
        Data3: ctypes._CField[Incomplete, Incomplete, Incomplete]
        Data4: ctypes._CField[Incomplete, Incomplete, Incomplete]

    class SP_DEVINFO_DATA(ctypes.Structure):
        cbSize: ctypes._CField[Incomplete, Incomplete, Incomplete]
        ClassGuid: ctypes._CField[Incomplete, Incomplete, Incomplete]
        DevInst: ctypes._CField[Incomplete, Incomplete, Incomplete]
        Reserved: ctypes._CField[Incomplete, Incomplete, Incomplete]

    PSP_DEVINFO_DATA: type[ctypes._Pointer[SP_DEVINFO_DATA]]
    PSP_DEVICE_INTERFACE_DETAIL_DATA = ctypes.c_void_p
    setupapi: ctypes.WinDLL
    SetupDiDestroyDeviceInfoList: ctypes._NamedFuncPointer
    SetupDiClassGuidsFromName: ctypes._NamedFuncPointer
    SetupDiEnumDeviceInfo: ctypes._NamedFuncPointer
    SetupDiGetClassDevs: ctypes._NamedFuncPointer
    SetupDiGetDeviceRegistryProperty: ctypes._NamedFuncPointer
    SetupDiGetDeviceInstanceId: ctypes._NamedFuncPointer
    SetupDiOpenDevRegKey: ctypes._NamedFuncPointer
    advapi32: ctypes.WinDLL
    RegCloseKey: ctypes._NamedFuncPointer
    RegQueryValueEx: ctypes._NamedFuncPointer
    cfgmgr32: ctypes.WinDLL
    CM_Get_Parent: ctypes._NamedFuncPointer
    CM_Get_Device_IDW: ctypes._NamedFuncPointer
    CM_MapCrToWin32Err: ctypes._NamedFuncPointer
    DIGCF_PRESENT: int
    DIGCF_DEVICEINTERFACE: int
    INVALID_HANDLE_VALUE: int
    ERROR_INSUFFICIENT_BUFFER: int
    ERROR_NOT_FOUND: int
    SPDRP_HARDWAREID: int
    SPDRP_FRIENDLYNAME: int
    SPDRP_LOCATION_PATHS: int
    SPDRP_MFG: int
    DICS_FLAG_GLOBAL: int
    DIREG_DEV: int
    KEY_READ: int
    MAX_USB_DEVICE_TREE_TRAVERSAL_DEPTH: int

    def get_parent_serial_number(
        child_devinst: ctypes._CData,
        child_vid: int | None,
        child_pid: int | None,
        depth: int = 0,
        last_serial_number: str | None = None,
    ) -> str: ...
    def iterate_comports() -> Generator[ListPortInfo]: ...
    def comports(include_links: bool = False) -> list[ListPortInfo]: ...



from pyasn1.type.namedtype import NamedTypes
from pyasn1.type.tag import TagSet
from pyasn1.type.univ import Sequence

class SicilyBindResponse(Sequence):
    tagSet: TagSet
    componentType: NamedTypes

class DirSyncControlRequestValue(Sequence):
    componentType: NamedTypes

class DirSyncControlResponseValue(Sequence):
    componentType: NamedTypes

class SdFlags(Sequence):
    componentType: NamedTypes

class ExtendedDN(Sequence):
    componentType: NamedTypes

def dir_sync_control(criticality, object_security, ancestors_first, public_data_only, incremental_values, max_length, cookie): ...
def extended_dn_control(criticality: bool = False, hex_format: bool = False): ...
def show_deleted_control(criticality: bool = False): ...
def security_descriptor_control(criticality: bool = False, sdflags: int = 15): ...
def persistent_search_control(criticality: bool = False): ...
